@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_client_types_h =
- *   "$Id: client-types.h,v 1.8 2005/05/06 19:55:43 tchize Exp $";
+ *   "$Id: client-types.h,v 1.9 2005/09/21 13:04:59 qal21 Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -80,8 +80,8 @@ typedef signed long         sint64;
 typedef unsigned long long      uint64;
 typedef signed long long        sint64;
 #else
-error do not know how to get a 64 bit value on this system.
-error correct and send mail to crossfire-devel on how to do this
+#error do not know how to get a 64 bit value on this system.
+#error correct and send mail to crossfire-devel on how to do this.
 #endif
 
 #else
