@@ -1,5 +1,5 @@
-char *rcsid_common_client_c =
-    "$Id: client.c,v 1.26 2005/05/21 17:41:48 tchize Exp $";
+const char *rcsid_common_client_c =
+    "$Id: client.c,v 1.34 2006/05/21 10:32:55 akirschbaum Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -33,9 +33,9 @@ char *rcsid_common_client_c =
   *
   * The DoClient function recieves a message (an ArgList), unpacks it, and
   * in a slow for loop dispatches the command to the right function through
-  * the commands table.   ArgLists are essentially like RPC things, only 
+  * the commands table.   ArgLists are essentially like RPC things, only
   * they don't require going through RPCgen, and it's easy to get variable
-  * length lists.  They are just lists of longs, strings, characters, and 
+  * length lists.  They are just lists of longs, strings, characters, and
   * byte arrays that can be converted to a machine independent format
  */
 
@@ -45,6 +45,8 @@ char *rcsid_common_client_c =
 #include <errno.h>
 #include <script.h>
 
+#include "mapdata.h"
+
 /* actually declare the globals */
 
 char *server=SERVER,*client_libdir=NULL,*meta_server=META_SERVER;
@@ -54,21 +56,23 @@ char *skill_names[MAX_SKILL];
 int meta_port=META_PORT, want_skill_exp=0,
     replyinfo_status=0, requestinfo_sent=0, replyinfo_last_face=0,
     maxfd,map1cmd=0,metaserver_on=METASERVER;
+uint32	tick=0;
+
 FILE *fpin,*fpout;
 Client_Player cpl;
 ClientSocket csocket;
 
-char *resists_name[NUM_RESISTS] = {
-"armor", "magic", "fire", "elec", 
+const char *const resists_name[NUM_RESISTS] = {
+"armor", "magic", "fire", "elec",
 "cold", "conf", "acid", "drain",
 "ghit", "pois", "slow", "para",
-"t undead", "fear", "depl","death", 
+"t undead", "fear", "depl","death",
 "hword", "blind"};
 
 typedef void (*CmdProc)(unsigned char *, int len);
 
 struct CmdMapping {
-  char *cmdname;
+  const char *cmdname;
   void (*cmdproc)(unsigned char *, int );
   enum CmdFormat cmdformat;
 };
@@ -78,6 +82,7 @@ struct CmdMapping commands[] = {
     /* Order of this table doesn't make a difference.  I tried to sort
      * of cluster the related stuff together.
      */
+    { "map2", Map2Cmd, SHORT_ARRAY },
     { "map1", Map1Cmd, SHORT_ARRAY },
     { "map1a", Map1aCmd, SHORT_ARRAY },
     { "map_scroll", (CmdProc)map_scrollCmd, ASCII },
@@ -91,6 +96,10 @@ struct CmdMapping commands[] = {
     { "delitem", DeleteItem, INT_ARRAY },
     { "delinv",	DeleteInventory, ASCII },
 
+    { "addspell", AddspellCmd, MIXED },
+    { "updspell", UpdspellCmd, MIXED },
+    { "delspell", DeleteSpell, INT_ARRAY },
+
     { "drawinfo", (CmdProc)DrawInfoCmd, ASCII },
     { "drawextinfo", (CmdProc)DrawExtInfoCmd, ASCII},
     { "stats", StatsCmd, STATS /* array of: int8, (int?s for that stat) */ },
@@ -100,6 +109,7 @@ struct CmdMapping commands[] = {
     { "face", FaceCmd, MIXED /* int16, string */},
     { "face1", Face1Cmd, MIXED /* int16, int32, string */},
     { "face2", Face2Cmd, MIXED /* int16, int8, int32, string */},
+    { "tick", TickCmd, INT_ARRAY /* uint32 */},
 
 
     { "sound", SoundCmd, MIXED /* int8, int8, int16, int8 */},
@@ -132,21 +142,23 @@ void DoClient(ClientSocket *csocket)
 	i=SockList_ReadPacket(csocket->fd, &csocket->inbuf, MAXSOCKBUF-1);
 	if (i==-1) {
 	    /* Need to add some better logic here */
-	    /*ET: not an error.  It's EOF!  At least errno isn't valid.
-	    fprintf(stderr,"Got error on read (error %d)\n", errno);
-	    */
+#ifdef WIN32
+	    closesocket(csocket->fd);
+#else
+	    close(csocket->fd);
+#endif
 	    csocket->fd=-1;
 	    return;
 	}
 	if (i==0) return;   /* Don't have a full packet */
-	csocket->inbuf.buf[csocket->inbuf.len]='\0';    
+	/* Terminate the buffer */
+	csocket->inbuf.buf[csocket->inbuf.len]='\0';
         data = (unsigned char *)strchr((char*)csocket->inbuf.buf +2, ' ');
 	if (data) {
 	    *data='\0';
 	    data++;
 	}
         len = csocket->inbuf.len - (data - csocket->inbuf.buf);
-	/* Terminate the buffer */
 	LOG(0,"Common::DoClient","Command:%s (%d)",csocket->inbuf.buf+2, len);
 	for(i=0;i < NCOMMANDS;i++) {
 	    if (strcmp((char*)csocket->inbuf.buf+2,commands[i].cmdname)==0) {
@@ -176,9 +188,10 @@ void DoClient(ClientSocket *csocket)
 
 int init_connection(char *host, int port)
 {
-    struct protoent *protox;
-    int fd, oldbufsize, newbufsize=65535, buflen=sizeof(int);
+    int fd = -1, oldbufsize, newbufsize=65535, buflen=sizeof(int);
+#if !HAVE_GETADDRINFO || WIN32
     struct sockaddr_in insock;
+    struct protoent *protox;
 
     protox = getprotobyname("tcp");
     if (protox == (struct protoent  *) NULL)
@@ -211,6 +224,45 @@ int init_connection(char *host, int port)
 	    perror("Can't connect to server");
 	    return -1;
     }
+#else
+    struct addrinfo hints;
+    struct addrinfo *res = NULL, *ai;
+    char port_str[6];
+
+    snprintf(port_str, sizeof(port_str), "%d", port);
+
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_protocol = IPPROTO_TCP;
+    
+    if (getaddrinfo(host, port_str, &hints, &res) != 0)
+	return -1;
+
+    for (ai = res; ai != NULL; ai = ai->ai_next) {
+	fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+	if (fd == -1)
+	    continue;
+
+	if (connect(fd, ai->ai_addr, ai->ai_addrlen) != 0) {
+	    close(fd);
+	    fd = -1;
+	    continue;
+	}
+
+	break;
+    }
+
+    freeaddrinfo(res);
+    if (fd == -1)
+	return -1;
+#endif
+
+    if (csocket.servername != NULL)
+        free(csocket.servername);
+    csocket.servername = malloc(sizeof(char)*(strlen(host)+1));
+    strcpy(csocket.servername, host);
+
 #ifndef WIN32
     if (fcntl(fd, F_SETFL, O_NDELAY)==-1) {
 	LOG (LOG_ERROR,"common::init_connection","Error on fcntl.");
@@ -299,13 +351,14 @@ void negotiate_connection(int sound)
      */
     if (face_info.want_faceset) face_info.faceset = atoi(face_info.want_faceset);
     cs_print_string(csocket.fd,
-	    "setup map1acmd 1 sound %d sexp %d darkness %d newmapcmd 1 faceset %d facecache %d exp64 1 itemcmd 2",
-	    sound>=0, want_skill_exp, 
+	    "setup map2cmd 1 tick 1 sound %d sexp %d darkness %d newmapcmd 1 spellmon 1 faceset %d facecache %d exp64 1 itemcmd 2",
+	    sound>=0, want_skill_exp,
 		    want_config[CONFIG_LIGHTING]?1:0, face_info.faceset,
 		    want_config[CONFIG_CACHE]);
 
     use_config[CONFIG_MAPHEIGHT]=want_config[CONFIG_MAPHEIGHT];
     use_config[CONFIG_MAPWIDTH]=want_config[CONFIG_MAPWIDTH];
+    mapdata_set_size(use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
     if (use_config[CONFIG_MAPHEIGHT]!=11 || use_config[CONFIG_MAPWIDTH]!=11)
 	cs_print_string(csocket.fd,"setup mapsize %dx%d",use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 
