@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_client_h =
- *   "$Id: client.h,v 1.30 2005/05/30 21:01:57 ryo_saeba Exp $";
+ *   "$Id: client.h,v 1.41 2006/05/15 05:57:43 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -38,13 +38,14 @@
 #  include <dmalloc.h>
 #endif
 
+#define MULTKEYS
 
 #define VERSION_CS 1023
 #define VERSION_SC 1027
 
 char VERSION_INFO[256];
 
-/* Don't send more than this many outstanding commands to the server 
+/* Don't send more than this many outstanding commands to the server
  * this is only a default value.
  */
 #define COMMAND_WINDOW 10
@@ -70,6 +71,9 @@ typedef struct Animations {
 			     * only faces[0],[1] have meaningfull values.
 			     */
     uint16  *faces;
+    uint8   speed;
+    uint8   speed_left;
+    uint8   phase;
 } Animations;
 
 extern Animations animations[MAXANIM];
@@ -95,7 +99,7 @@ typedef struct ClientSocket {
     int command_sent, command_received;
     /* Time (in ms) players commands currently take to execute */
     int command_time;
-	    
+    char* servername;
 } ClientSocket;
 
 extern ClientSocket csocket;
@@ -113,7 +117,7 @@ typedef enum rangetype {
 } rangetype;
 
 /* This is a structure that contains most all of the
- * configuration options.  Instead of having a 
+ * configuration options.  Instead of having a
  * whole mess of variables of different names, instead use
  * a common 16 bit signed array, and index into these -
  * this makes processing in the gui aspect of the GTK
@@ -121,59 +125,61 @@ typedef enum rangetype {
  * want options, and use_options.  The former is what the
  * player wants to use, the later is what is currently
  * in use.  There are many options that can not be
- * switched between during actual play, but we want to 
- * record what the player has changed them to so that 
+ * switched between during actual play, but we want to
+ * record what the player has changed them to so that
  * when we save them out, we save what the player wants,
  * and not what is currently being used.  Note that all the gui
  * interfaces may not use all these values, but making them
- * available here makes it easy for the GUI to present a 
+ * available here makes it easy for the GUI to present a
  * nice interface.
  * 0 is intentially skipped so the index into this doesn't
  * get a default if a table has a blank value
  *
- * CONFIG_NUMS is the number of configuration options; don't 
+ * CONFIG_NUMS is the number of configuration options; don't
  * forget to add to some of
- *   common/init.c config_names, 
+ *   common/init.c config_names,
  *                 init_client_vars,
  *   x11/x11.c load_defaults
  *             save_defaults
  *   gtk/config.c load_defaults
  *                save_defaults
- * probably among other places, if you add a new option. 
+ * probably among other places, if you add a new option.
  */
-#define CONFIG_COLORINV	    1
-#define CONFIG_COLORTXT	    2
-#define CONFIG_DOWNLOAD	    3
-#define CONFIG_ECHO	    4
-#define CONFIG_FASTTCP	    5
-#define CONFIG_CWINDOW	    6
-#define CONFIG_CACHE	    7
-#define CONFIG_FOGWAR	    8
-#define CONFIG_ICONSCALE    9
-#define CONFIG_MAPSCALE	    10
-#define CONFIG_POPUPS	    11
-#define CONFIG_DISPLAYMODE  12
-#define CONFIG_SHOWICON	    13
-#define CONFIG_TOOLTIPS	    14
-#define CONFIG_SOUND	    15
-#define CONFIG_SPLITINFO    16
-#define CONFIG_SPLITWIN	    17
-#define CONFIG_SHOWGRID	    18
-#define CONFIG_LIGHTING	    19
-#define CONFIG_TRIMINFO	    20
-#define CONFIG_MAPWIDTH	    21
-#define CONFIG_MAPHEIGHT    22
-#define CONFIG_FOODBEEP	    23
-#define CONFIG_DARKNESS	    24
-#define CONFIG_PORT	    25		/* Not sure if useful at all anymore */
-#define CONFIG_GRAD_COLOR   26
-#define CONFIG_RESISTS      27
-#define CONFIG_SMOOTH       28
-#define CONFIG_SPLASH	    29
-#define CONFIG_APPLY_CONTAINER	30	/* Reapply container */
-#define CONFIG_NUMS	    31
+#define CONFIG_DOWNLOAD	    1
+#define CONFIG_ECHO	    2
+#define CONFIG_FASTTCP	    3
+#define CONFIG_CWINDOW	    4
+#define CONFIG_CACHE	    5
+#define CONFIG_FOGWAR	    6
+#define CONFIG_ICONSCALE    7
+#define CONFIG_MAPSCALE	    8
+#define CONFIG_POPUPS	    9
+#define CONFIG_DISPLAYMODE  10
+#define CONFIG_SHOWICON	    11
+#define CONFIG_TOOLTIPS	    12
+#define CONFIG_SOUND	    13
+#define CONFIG_SPLITINFO    14
+#define CONFIG_SPLITWIN	    15
+#define CONFIG_SHOWGRID	    16
+#define CONFIG_LIGHTING	    17
+#define CONFIG_TRIMINFO	    18
+#define CONFIG_MAPWIDTH	    19
+#define CONFIG_MAPHEIGHT    20
+#define CONFIG_FOODBEEP	    21
+#define CONFIG_DARKNESS	    22
+#define CONFIG_PORT	    23		/* Not sure if useful at all anymore */
+#define CONFIG_GRAD_COLOR   24
+#define CONFIG_RESISTS      25
+#define CONFIG_SMOOTH       26
+#define CONFIG_SPLASH	    27
+#define CONFIG_APPLY_CONTAINER	28	/* Reapply container */
+#define CONFIG_MAPSCROLL    29		/* Use bitmap operations for map scrolling */
+#define CONFIG_SIGNPOPUP    30
+#define CONFIG_TIMESTAMP    31
+#define CONFIG_NUMS	    32		/* This should always be the last value in the list */
 
 /* CONFIG_LIGHTING can have several possible values - set them accordingly */
+#define CFG_LT_NONE	    0
 #define CFG_LT_TILE	    1
 #define CFG_LT_PIXEL	    2
 #define CFG_LT_PIXEL_BEST   3
@@ -191,7 +197,7 @@ extern sint16 want_config[CONFIG_NUMS], use_config[CONFIG_NUMS];
 /* see common/init.c - basically, this is a string to number
  * mapping that is used when loading/saving the values.
  */
-extern char *config_names[CONFIG_NUMS];
+extern const char *const config_names[CONFIG_NUMS];
 
 
 typedef struct Stat_struct {
@@ -209,6 +215,9 @@ typedef struct Stat_struct {
     sint16 dam;		    /* How much damage this object does when hitting */
     sint32 speed;	    /* Gets converted to a float for display*/
     sint32 weapon_sp;	    /* Gets converted to a float for display */
+    uint32 attuned;         /* The spell paths to which the player is attuned */
+    uint32 repelled;        /* The spell paths to which the player is repelled */
+    uint32 denied;          /* The spell paths denied to the player*/
     uint16 flags;	    /* contains fire on/run on flags */
     sint16 resists[30];	    /* Resistant values */
     uint32 resist_change:1; /* Resistant value has changed */
@@ -218,6 +227,22 @@ typedef struct Stat_struct {
 } Stats;
 
 
+typedef struct Spell_struct {
+    struct Spell_struct *next;
+    char name[256]; /* the protocol allows one length bit, so 256 is the maximum name length */
+    char message[10000]; /* this is plenty, the packets can't be much bigger than this anyway */
+    uint32 tag; /* used to identify the spell by updspell */
+    uint16 level;
+    uint16 time; /* number of ticks to cast */
+    uint16 sp;
+    uint16 grace;
+    uint16 dam;
+    uint8 skill_number; /* the index in the skill arrays, plus CS_STAT_SKILLINFO */
+    char *skill; /* pointer to the skill name, derived from the skill number */
+    uint32 path; /* the bitmask of paths this spell belongs to */
+    sint32 face; 
+} Spell;
+
 typedef struct Player_Struct {
     item	*ob;		/* Player object */
     item	*below;		/* Items below the player (pl.below->inv) */
@@ -232,10 +257,14 @@ typedef struct Player_Struct {
     char	spells[255][40];	/* List of all the spells the */
 				/* player knows */
     Stats	stats;		/* Player stats */
+    Spell	*spelldata;	/* linked list of spells known to the player */
     char	title[MAX_BUF];	/* Title of character */
     char	range[MAX_BUF];	/* Range attack chosen */
+    uint32	spells_updated; /* tracks whether the spells updated*/ 
     uint32	fire_on:1;	/* True if fire key is pressed */
     uint32	run_on:1;	/* True if run key is on */
+    uint32	meta_on:1;	/* True if fire key is pressed */
+    uint32	alt_on:1;	/* True if fire key is pressed */
     uint32	no_echo:1;	/* If TRUE, don't echo keystrokes */
     uint32	count;		/* Repeat count on command */
     uint16	mmapx, mmapy;	/* size of magic map */
@@ -247,7 +276,7 @@ typedef struct Player_Struct {
     uint16	mapxres,mapyres;/* Resolution to draw on the magic map
                                  * Only used in client-specific code, so it should move there. */
 
-#ifdef WIN32
+#ifdef MULTKEYS
     char    name[ 40 ]; /* Player's name, for player-specific key files */
 #endif
 } Client_Player;
@@ -296,7 +325,7 @@ typedef struct {
 } Face_Information;
 
 extern Face_Information face_info;
-    
+
 
 extern Client_Player cpl;		/* Player object. */
 extern char *skill_names[MAX_SKILL];
@@ -352,10 +381,11 @@ typedef struct ChildProcess{
  * in matching order.
  */
 #define NUM_RESISTS 18
-extern char *resists_name[NUM_RESISTS];
+extern const char *const resists_name[NUM_RESISTS];
 extern char *meta_server;
 extern int meta_port,want_skill_exp;
 extern int map1cmd,metaserver_on;
+extern uint32	tick;
 
 /* Map size the client will request the map to be.  Bigger it is,
  * more memory it will use
@@ -376,11 +406,7 @@ extern int map1cmd,metaserver_on;
  * the map edge.  This is the most it may be off.  This is needed
  * To cover case of need_recenter_map routines.
  */
-#define MAX_MAP_OFFSET	6
-
-/* Fog of war stuff */
-#define FOG_MAP_SIZE 512   /* Default size of virtual map */
-
+#define MAX_MAP_OFFSET	8
 
 /* Start of map handling code.
  * For the most part, this actually is not window system specific,
@@ -389,55 +415,10 @@ extern int map1cmd,metaserver_on;
 
 #define MAXPIXMAPNUM 10000
 
-/* The protocol only supports 3 layers, so set MAXLAYERS accordingly.
- * the heads[] in the mapcell is used for single part objects
- * or the head piece for multipart.  The easiest way to think about
- * it is that the heads[] contains the map information as specifically
- * sent from the server.  for the heads value, the size_x and size_y
- * represent how many spaces (up and to the left) that image extends
- * into.
- * the tails are values that the client fills in - if we get
- * a big head value, we fill in the tails value so that the display
- * logic can easily redraw one space.  In this case, the size_ values
- * are offsets that point to the head.  In this way, the draw logic
- * can look at the size of the image, look at these values, and
- * know what portion of it to draw.
- */
-
-#define MAXLAYERS 3
-
-struct MapCellLayer {
-    sint16  face;
-    sint8   size_x;
-    sint8   size_y;
-};
-
-struct MapCell {
-    struct MapCellLayer	heads[MAXLAYERS];
-    struct MapCellLayer	tails[MAXLAYERS];
-    uint16 smooth[MAXLAYERS];
-    uint8 darkness;
-    uint8 need_update:1;
-    uint8 have_darkness:1;
-    uint8 need_resmooth:1;  /*same has need update but for smoothing only*/
-    uint8 cleared:1; /* If set, this is a fog cell. */
-    uint8 keephead:1;
-};
-
-
-struct Map {
-  struct MapCell **cells;
-  /* Store size of map so we know if map_size has changed
-   * since the last time we allocated this;
-   */
-  int x;
-  int y;
-};
-
 /* This is used mostly in the cache.c file, however, it
  * can be returned to the graphic side of things so that they
  * can update the image_data field.  Since the common side
- * has no idea what data the graphic side will point to, 
+ * has no idea what data the graphic side will point to,
  * we use a void pointer for that - it is completely up to
  * the graphic side to allocate/deallocate and cast that
  * pointer as needed.
@@ -445,12 +426,12 @@ struct Map {
 typedef struct Cache_Entry {
     char    *filename;
     uint32  checksum;
-    uint32  public:1;
+    uint32  ispublic:1;
     void    *image_data;
     struct Cache_Entry	*next;
 } Cache_Entry;
 
-/* These values are used for various aspects of the library to 
+/* These values are used for various aspects of the library to
  * hold state on what requestinfo's we have gotten replyinfo
  * for and what data was received.  In this way, common/client.c
  * can loop until it has gotten replies for all the requestinfos
@@ -465,7 +446,7 @@ typedef struct Cache_Entry {
 #define RI_IMAGE_SUMS	    0x2
 extern int  replyinfo_status, requestinfo_sent, replyinfo_last_face;
 
-typedef struct 
+typedef struct
 {
   int x;
   int y;
@@ -473,10 +454,6 @@ typedef struct
 
 extern PlayerPosition pl_pos;
 
-extern struct Map the_map;
-
-
-
 extern TextManager* firstTextManager;
 
 #ifndef MIN
