@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_external_h =
- *   "$Id: external.h,v 1.9 2005/02/27 06:06:29 mwedel Exp $";
+ *   "$Id: external.h,v 1.12 2006/02/16 22:32:45 akirschbaum Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -40,9 +40,8 @@ extern void resize_map_window(int x, int y);
 extern void display_map_newmap(void);
 extern void display_map_addbelow(long x, long y, long face);
 extern void display_map_startupdate(void);
-extern void display_map_doneupdate(int redraw);
-extern void set_map_darkness(int x, int y, uint8 darkness);
-extern void display_mapscroll(int dx, int dy);
+extern void display_map_doneupdate(int redraw, int notice);
+extern int display_mapscroll(int dx, int dy);
 extern void draw_magic_map(void);
 
 /* Info related functions */
@@ -52,8 +51,8 @@ extern void draw_prompt(const char *str);
 extern void x_set_echo(void);
 extern void set_scroll(const char *s);
 extern void set_autorepeat(const char *s);
-extern void menu_clear();
-extern int get_info_width();
+extern void menu_clear(void);
+extern int get_info_width(void);
 
 /* Stats related commands */
 extern void draw_stats(int redraw);
@@ -70,6 +69,7 @@ extern uint8 *png_to_data(uint8 *data, int len, uint32 *width, uint32 *height);
 extern int associate_cache_entry(Cache_Entry *ce, int pixnum);
 extern void image_update_download_status(int start, int end, int total);
 extern void get_map_image_size(int face, uint8 *w, uint8 *h);
+extern void addsmooth(uint16 face, uint16 smooth_face);
 
 /* Item related commands */
 extern void open_container(item *op);
