@@ -1,5 +1,5 @@
-char *rcsid_common_image_c =
-    "$Id: image.c,v 1.18 2005/03/27 20:51:24 akirschbaum Exp $";
+const char *rcsid_common_image_c =
+    "$Id: image.c,v 1.22 2006/04/12 11:18:00 cavesomething Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -143,7 +143,7 @@ static int load_image(char *filename, uint8 *data, int *len, int *csum)
 	    ROTATE_RIGHT(*csum);
 	    *csum += data[i];
 	    *csum &= 0xffffffff;
-p	}
+	}
 #endif
 
 }
@@ -161,8 +161,8 @@ p	}
  *******************************************************************************
  */
 
-/* This should be a prime */
-#define IMAGE_HASH  5399
+/* This should be a power of 2 */
+#define IMAGE_HASH  8192
 
 Face_Information face_info;
 
@@ -180,7 +180,7 @@ struct Image_Cache {
 } image_cache[IMAGE_HASH];
 
 
-/* This function is basically hasarch from the server, common/arch.c
+/* This function is basically hasharch from the server, common/arch.c
  * a few changes - first, we stop processing when we reach the first
  * . - this is because I'm not sure if hashing .111 at the end of
  * all the image names will be very useful.
@@ -188,16 +188,18 @@ struct Image_Cache {
 
 static uint32 image_hash_name(char *str, int tablesize) {
     uint32 hash = 0;
-    int rot = 0;
     char *p;
 
+    /* use the same one-at-a-time hash function the server now uses */
     for (p = str; *p!='\0' && *p != '.'; p++) {
-        hash ^= (uint32) *p << rot;
-        rot += 2;
-        if (rot >= (int)(sizeof(uint32) - sizeof(char)) * 8)
-            rot = 0;
+        hash += *p;
+        hash += hash << 10;
+        hash ^= hash >>  6;
     }
-    return (hash % tablesize);
+    hash += hash <<  3;
+    hash ^= hash >> 11;
+    hash += hash << 15;
+    return hash % tablesize;
 }
 
 /* This function returns an index into the image_cache for
@@ -275,7 +277,7 @@ static Cache_Entry *image_find_cache_entry(char *imagename, uint32 checksum, int
 }
 
 /* Add a hash entry.  Returns the entry we added, NULL on failure.. */
-static Cache_Entry *image_add_hash(char *imagename, char *filename, uint32 checksum, uint32 public)
+static Cache_Entry *image_add_hash(char *imagename, char *filename, uint32 checksum, uint32 ispublic)
 {
     Cache_Entry *new_entry;
     uint32  hash = image_hash_name(imagename, IMAGE_HASH), newhash;
@@ -303,7 +305,7 @@ static Cache_Entry *image_add_hash(char *imagename, char *filename, uint32 check
     new_entry = malloc(sizeof(struct Cache_Entry));
     new_entry->filename = strdup(filename);
     new_entry->checksum = checksum;
-    new_entry->public = public;
+    new_entry->ispublic = ispublic;
     new_entry->image_data = NULL;
     new_entry->next = image_cache[newhash].cache_entry;
     image_cache[newhash].cache_entry = new_entry;
@@ -315,7 +317,7 @@ static Cache_Entry *image_add_hash(char *imagename, char *filename, uint32 check
  * but we try to be lenient/follow some conventions.
  * Note that this is destructive to the data passed in line.
  */
-static void image_process_line(char *line, uint32 public)
+static void image_process_line(char *line, uint32 ispublic)
 {
     char imagename[MAX_BUF], filename[MAX_BUF];
     uint32 checksum;
@@ -323,7 +325,7 @@ static void image_process_line(char *line, uint32 public)
     if (line[0] == '#') return;		    /* Ignore comments */
 
     if (sscanf(line, "%s %u %s", imagename, &checksum, filename)==3) {
-	image_add_hash(imagename, filename, checksum, public);
+	image_add_hash(imagename, filename, checksum, ispublic);
     } else {
 	LOG(LOG_WARNING,"common::image_process_line","Did not parse line %s properly?", line);
     }
@@ -430,7 +432,7 @@ void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face, int fac
 	    /* If this has image_data, then it has already been rendered */
 	    if (!associate_cache_entry(ce, pnum)) return;
 	}
-	if (ce->public)
+	if (ce->ispublic)
 	    sprintf(filename,"%s/%s",
 		    DATADIR, ce->filename);
 	else
@@ -449,7 +451,7 @@ void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face, int fac
 	/* If the data is bad, remove it if it is in the players private cache */
 	LOG(LOG_WARNING,"common::finish_face_cmd","Got error on png_to_data, image=%s",face);
 	if (ce) {
-	    if (!ce->public) unlink(filename);
+	    if (!ce->ispublic) unlink(filename);
 	    image_remove_hash(face,ce);
 	}
 
