@@ -1,5 +1,5 @@
-char *rcsid_common_init_c =
-    "$Id: init.c,v 1.22 2005/07/10 10:19:02 ryo_saeba Exp $";
+const char *rcsid_common_init_c =
+    "$Id: init.c,v 1.31 2006/05/15 05:57:43 mwedel Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -37,14 +37,14 @@ char *rcsid_common_init_c =
  * something like a fprintf(outifle, "%s: %d", config_names[i],
  *			    want_config[i]);
  */
-char *config_names[CONFIG_NUMS] = {
-NULL, "colorinv", "colortext", "download_all_images", "echo_bindings",
+const char *const config_names[CONFIG_NUMS] = {
+NULL, "download_all_images", "echo_bindings",
 "fasttcpsend", "command_window", "cacheimages", "fog_of_war", "iconscale",
-"mapscale", "popups", "sdl", "showicon", "tooltips", "sound", "splitinfo",
+"mapscale", "popups", "displaymode", "showicon", "tooltips", "sound", "splitinfo",
 "split", "show_grid", "lighting", "trim_info_window",
 "map_width", "map_height", "foodbeep", "darkness", "port",
-"grad_color_bars", "resists", "smoothing", "nosplash",
-"auto_apply_container"
+"grad_color_bars", "resistances", "smoothing", "nosplash",
+"auto_apply_container", "mapscroll", "sign_popups", "message_timestamping"
 };
 
 sint16 want_config[CONFIG_NUMS], use_config[CONFIG_NUMS];
@@ -161,8 +161,6 @@ void init_client_vars()
      * the code.  Plus, the performance difference is virtually
      * nothing.
      */
-    want_config[CONFIG_COLORINV] = TRUE;
-    want_config[CONFIG_COLORTXT] = TRUE;
     want_config[CONFIG_DOWNLOAD] = FALSE;
     want_config[CONFIG_ECHO] = FALSE;
     want_config[CONFIG_FASTTCP] = TRUE;
@@ -192,6 +190,9 @@ void init_client_vars()
     want_config[CONFIG_SMOOTH] = 0;
     want_config[CONFIG_SPLASH] = TRUE;
     want_config[CONFIG_APPLY_CONTAINER] = TRUE;	    /* Same behavior before this option was put in */
+    want_config[CONFIG_MAPSCROLL] = TRUE;
+    want_config[CONFIG_SIGNPOPUP] = TRUE;
+    want_config[CONFIG_TIMESTAMP] = FALSE;
     for (i=0; i<CONFIG_NUMS; i++) 
 	use_config[i] = want_config[i];
 
@@ -214,6 +215,9 @@ void init_client_vars()
         }
 #endif
     init_commands(); /* pcmd.c */
+
+    /* Any reasonable seed really works */
+    srandom(time(NULL));
 }
 
 /* This is basically called each time a new player logs
