@@ -1,5 +1,5 @@
-char *rcsid_common_item_c =
-    "$Id: item.c,v 1.12 2005/06/04 20:56:21 mwedel Exp $";
+const char *rcsid_common_item_c =
+    "$Id: item.c,v 1.16 2006/03/14 18:08:45 ryo_saeba Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -41,7 +41,7 @@ static item *player, *map;	/* these lists contains rest of items */
 
 #include <item-types.h>
 /* This should be modified to read the definition from a file */
-void init_item_types()
+void init_item_types(void)
 {
 }
 
@@ -187,18 +187,23 @@ void update_item_sort(item *it)
  * It is currently only used by the query_name() function.
  */
 
-char *get_number(int i) {
+char *get_number(uint32 i) {
 static char numbers[21][20] = {
   "no","a","two","three","four","five","six","seven","eight","nine","ten",
   "eleven","twelve","thirteen","fourteen","fifteen","sixteen","seventeen",
   "eighteen","nineteen","twenty"
 };
+  static char buf[MAX_BUF];
 
+  if(i<0)
+  {
+	  sprintf(buf,"negative");
+	  return buf;
+  }
 
   if(i<=20)
     return numbers[i];
   else {
-    static char buf[MAX_BUF];
     sprintf(buf,"%d",i);
     return buf;
   }
@@ -209,7 +214,7 @@ static char numbers[21][20] = {
  *  new_item() returns pointer to new item which
  *  is allocated and initialized correctly
  */
-static item *new_item () 
+static item *new_item(void)
 {
     item *op = malloc (sizeof(item));
 
@@ -234,6 +239,7 @@ static item *new_item ()
     op->nrof=0;
     op->open=0;
     op->type=NO_ITEM_TYPE;
+    op->inv_updated = 0;
     return op;
 }
 
@@ -436,7 +442,7 @@ int num_free_items()
 /*
  *  Hardcoded now, server could send these at initiation phase.
  */
-static char *apply_string[] = {
+static const char *const apply_string[] = {
     "", " (readied)", " (wielded)", " (worn)", " (active)", " (applied)"
 };
 
@@ -483,7 +489,7 @@ static void get_flags (item *op, uint16 flags)
 
 void set_item_values (item *op, char *name, sint32 weight, uint16 face, 
 		      uint16 flags, uint16 anim, uint16 animspeed,
-		      sint32 nrof, uint16 type) 
+		      uint32 nrof, uint16 type) 
 {
     int resort=1;
 
@@ -591,7 +597,7 @@ item *map_item ()
 
 /* Upates an item with new attributes. */
 void update_item(int tag, int loc, char *name, int weight, int face, int flags,
-		 int anim, int animspeed, int nrof, int type)
+		 int anim, int animspeed, uint32 nrof, int type)
 {
     item *ip = locate_item(tag), *env=locate_item(loc);
 
