@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_h =
- *   "$Id: item.h,v 1.4 2005/05/30 21:01:57 ryo_saeba Exp $";
+ *   "$Id: item.h,v 1.5 2005/12/27 14:26:29 ryo_saeba Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -52,7 +52,7 @@ typedef struct item_struct {
     char p_name[NAME_LEN];	/* item's plural name as sent to us */
     char flags[NAME_LEN];	/* item's status information */
     sint32 tag;			/* item identifier (0 = free) */
-    sint32 nrof;		/* number of items */
+    uint32 nrof;		/* number of items */
     float weight;		/* how much item weights */
     sint16 face;		/* index for face array */
     uint16 animation_id;	/* Index into animation array */
