@@ -1,5 +1,5 @@
-char *rcsid_common_misc_c =
-    "$Id: misc.c,v 1.10 2005/03/28 06:44:53 mwedel Exp $";
+const char *rcsid_common_misc_c =
+    "$Id: misc.c,v 1.14 2006/02/23 20:18:28 akirschbaum Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -24,7 +24,7 @@ char *rcsid_common_misc_c =
 
 /*
  * static char *rcsid_misc_c =
- *   "$Id: misc.c,v 1.10 2005/03/28 06:44:53 mwedel Exp $";
+ *   "$Id: misc.c,v 1.14 2006/02/23 20:18:28 akirschbaum Exp $";
  */
 
 
@@ -41,7 +41,6 @@ char *rcsid_common_misc_c =
 #ifndef WIN32
 #include <sys/wait.h>
 #else
-#define inline __inline
 #include <direct.h>
 #include <io.h>
 #endif
@@ -138,26 +137,28 @@ int setLogListener(LogListener li){
     loglist=li;
     return 1;
 }
-void clearLogListener(){
+void clearLogListener(void) {
     loglist=NULL;
 }
-static char* LogLevelTexts[]={" DEBUG  ",
-                              "  INFO  ",
-                              "WARNING ",
-                              " ERROR  ",
-                              "CRITICAL",
-                              "UNKNOWN "};
-static inline char * getLogLevelText(LogLevel level){
+static const char *const LogLevelTexts[] = {
+    " DEBUG  ",
+    "  INFO  ",
+    "WARNING ",
+    " ERROR  ",
+    "CRITICAL",
+    "UNKNOWN ",
+};
+static const char *getLogLevelText(LogLevel level) {
     return LogLevelTexts[level>LOG_CRITICAL?LOG_CRITICAL+1:level];
 }
-char* getLogTextRaw(LogLevel level, char* origin, char*message){
+char *getLogTextRaw(LogLevel level, const char *origin, const char *message) {
     static char mybuf[20480];
     mybuf[0]='\0';
     sprintf(mybuf,"[%s] (%s) %s\n",getLogLevelText(level),origin,message);
     return mybuf;
 }
 
-char* getLogText(LogEntry* le){
+char *getLogText(const LogEntry *le) {
     return getLogTextRaw(le->level,le->origin,le->message);
 }
 /*
@@ -172,11 +173,11 @@ char* getLogText(LogEntry* le){
 #else
 #define MINLOG 1
 #endif
-void LOG (LogLevel level, char* origin, char *format, ...)
+void LOG(LogLevel level, const char *origin, const char *format, ...)
 {
 
   va_list ap;
-  char buf[20480];  /* This needs to be really really big - larger
+  static char buf[20480];  /* This needs to be really really big - larger
 		     * than any other buffer, since that buffer may
 		     * need to be put in this one.
 		     */
@@ -187,7 +188,7 @@ void LOG (LogLevel level, char* origin, char *format, ...)
 
   buf[0] = '\0';
   vsprintf(buf, format, ap);
-  //fprintf(stderr,getLogTextRaw(level,origin,buf));
+  /*fprintf(stderr,getLogTextRaw(level,origin,buf));*/
   if (strlen(buf)>0){
     LogEntry *le = LOG_NEW_ENTRY;
     LOG_APPEND(le);
@@ -233,7 +234,7 @@ void purgePipe(ChildProcess* cp, int pipe){
     }
 }
 
-void monitorChilds(){
+void monitorChilds(void) {
 #ifndef WIN32
     ChildProcess* cp=FirstChild;
     ChildProcess* last=NULL;
