@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newclient_h =
- *   "$Id: newclient.h,v 1.6 2005/05/29 15:34:44 tchize Exp $";
+ *   "$Id: newclient.h,v 1.9 2006/05/21 10:32:55 akirschbaum Exp $";
  */
 
 /*
@@ -49,11 +49,18 @@
 #define NEWCLIENT_H
 
 /* Maximum size of any packet we expect.  Using this makes it so we don't need to
- * allocated and deallocated teh same buffer over and over again and the price
- * of using a bit of extra memory.  IT also makes the code simpler.
+ * allocate and deallocate the same buffer over and over again and the price
+ * of using a bit of extra memory.  It also makes the code simpler.
+ * The size is big enough to receive any valid packet: 2 bytes for length,
+ * 65535 for max. packet size, 1 for appended trailing '\0'.
  */
-#define MAXSOCKBUF 10240
+#define MAXSOCKBUF (2+65535+1)
 
+/* How much the x,y coordinates in the map2 are off from
+ * actual upper left corner.  Necessary for light sources
+ * that may be off the edge of the visible map.
+ */
+#define MAP2_COORD_OFFSET   15
 
 #define CS_QUERY_YESNO	0x1	/* Yes/no question */
 #define CS_QUERY_SINGLECHAR 0x2	/* Single character response expected */
@@ -99,6 +106,9 @@
 #define CS_STAT_FLAGS	25
 #define CS_STAT_WEIGHT_LIM	26
 #define CS_STAT_EXP64	28
+#define CS_STAT_SPELL_ATTUNE	29
+#define CS_STAT_SPELL_REPEL	30
+#define CS_STAT_SPELL_DENY	31
 
 /* Start & end of resistances, inclusive. */
 #define CS_STAT_RESIST_START	100
@@ -225,9 +235,24 @@ enum {a_none, a_readied, a_wielded, a_worn, a_active, a_applied};
 #define UPD_ANIMSPEED	0x40
 #define UPD_NROF	0x80
 
+/* The flag values sent by the updspell command */
+#define UPD_SP_MANA	0x01
+#define UPD_SP_GRACE	0x02
+#define UPD_SP_DAMAGE	0x04
+
 #define SOUND_NORMAL	0
 #define SOUND_SPELL	1
 
+/* Animation types */
+#define FACE_IS_ANIM	1<<15
+#define ANIM_RANDOM     1<<13
+#define ANIM_SYNC       2<<13
+
+#define ANIM_FLAGS_MASK	0x6000
+
+/* AND'ing this with data from server gets us just the animation id */
+#define ANIM_MASK	0x1fff
+
 /* Contains the base information we use to make up a packet we want to send. */
 typedef struct SockList {
     int len;
