@@ -1,5 +1,5 @@
-char *rcsid_common_newsocket_c =
-    "$Id: newsocket.c,v 1.13 2005/02/05 23:31:31 ryo_saeba Exp $";
+const char *rcsid_common_newsocket_c =
+    "$Id: newsocket.c,v 1.15 2006/05/21 10:32:55 akirschbaum Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -50,20 +50,17 @@ char *rcsid_common_newsocket_c =
 }*/
 
 
-/* We don't care what these values are in the client, since
- * we toss them
- */
-#define llevDebug 0
-#define llevError 0
+#define llevDebug LOG_DEBUG
+#define llevError LOG_ERROR
 
 
 /*
  * This writes data to the socket.
  */
-static int write_socket(int fd, unsigned char *buf, int len)
+static int write_socket(int fd, const unsigned char *buf, int len)
 {
     int amt=0;
-    unsigned char *pos=buf;
+    const unsigned char *pos=buf;
 
     /* If we manage to write more than we wanted, take it as a bonus */
     while (len>0) {
@@ -136,20 +133,20 @@ int SockList_Send(SockList *sl, int fd)
 }
 
 
-char GetChar_String(unsigned char *data)
+char GetChar_String(const unsigned char *data)
 {
     return (data[0]);
 }
 /* Basically does the reverse of SockList_AddInt, but on
  * strings instead.  Same for the GetShort, but for 16 bits.
  */
-int GetInt_String(unsigned char *data)
+int GetInt_String(const unsigned char *data)
 {
     return ((data[0]<<24) + (data[1]<<16) + (data[2]<<8) + data[3]);
 }
 
 /* 64 bit version of the above */
-sint64 GetInt64_String(unsigned char *data)
+sint64 GetInt64_String(const unsigned char *data)
 {
 #ifdef WIN32
     return (((sint64)data[0]<<56) + ((sint64)data[1]<<48) + 
@@ -162,15 +159,15 @@ sint64 GetInt64_String(unsigned char *data)
 #endif
 }
 
-short GetShort_String(unsigned char *data) {
+short GetShort_String(const unsigned char *data) {
     return ((data[0]<<8)+data[1]);
 }
 
-/* this readsfrom fd and puts the data in sl.  We return true if we think
- * we have a full packet, 0 if we have a partial packet.  The only processing
- * we do is remove the intial size value.  len (As passed) is the size of the
- * buffer allocated in the socklist.  We make the assumption the buffer is
- * at least 2 bytes long.
+/* This reads from fd and puts the data in sl. We return true if we think we
+ * have a full packet, 0 if we have a partial packet, or -1 if an error
+ * occurred. The only processing we do is remove the initial size value. len
+ * (As passed) is the size of the buffer allocated in the socklist. We make
+ * the assumption the buffer is at least 2 bytes long.
  */
  
 int SockList_ReadPacket(int fd, SockList *sl, int len)
