@@ -1,5 +1,5 @@
-char *rcsid_common_player_c =
-    "$Id: player.c,v 1.15 2005/02/10 07:03:36 mwedel Exp $";
+const char *rcsid_common_player_c =
+    "$Id: player.c,v 1.19 2006/02/23 20:18:28 akirschbaum Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -45,7 +45,7 @@ char *rcsid_common_player_c =
  * makes handling on the server side easier.
  */
 
-char *directions[9] = {"stay", "north", "northeast", "east", "southeast",
+const char *const directions[9] = {"stay", "north", "northeast", "east", "southeast",
 		"south","southwest", "west", "northwest"};
 
 
@@ -54,11 +54,22 @@ char *directions[9] = {"stay", "north", "northeast", "east", "southeast",
  */
 void new_player (long tag, char *name, long weight, long face)
 {
+    Spell *spell, *spnext;
+
     cpl.ob->tag    = tag;
     cpl.ob->nrof   = 1;
     copy_name (cpl.ob->d_name, name);
     cpl.ob->weight = (float) weight / 1000;
     cpl.ob->face   = face;
+
+    if (cpl.spelldata) {
+	for (spell = cpl.spelldata; spell; spell = spnext) {
+	    spnext = spell->next;
+	    free(spell);
+	}
+	cpl.spelldata = NULL;
+    }
+	
 }
 
 void look_at(int x, int y)
@@ -236,6 +247,7 @@ void CompleteCmd(unsigned char *data, int len)
 {
     if (len !=6) {
 	LOG(LOG_ERROR,"common::CompleteCmd","Invalid length %d - ignoring", len);
+	return;
     }
     csocket.command_received = GetShort_String(data);
     csocket.command_time = GetInt_String(data+2);
