@@ -1,5 +1,5 @@
-char *rcsid_common_script_c =
-    "$Id: script.c,v 1.13 2005/07/18 20:10:17 akirschbaum Exp $";
+const char *rcsid_common_script_c =
+    "$Id: script.c,v 1.20 2006/02/26 10:08:35 ryo_saeba Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -127,6 +127,7 @@ Someday this will be fixed :)
 #include <external.h>
 #include <script.h>
 #include <p_cmd.h>
+#include "mapdata.h"
 
 /*
  * Data structures
@@ -150,6 +151,7 @@ struct script {
    int pid;
 #else
    DWORD pid;	/* Handle to Win32 process ID */
+   HANDLE process; /* Handle of Win32 process */
 #endif
    int sync_watch;
 };
@@ -167,7 +169,7 @@ static int script_by_name(const char *name);
 static void script_dead(int i);
 static void script_process_cmd(int i);
 static void send_map(int i,int x,int y);
-static void script_send_item(int i,char *head,item *it);
+static void script_send_item(int i, const char *head, const item *it);
 
 
 /*
@@ -184,6 +186,7 @@ static int emulate_read(HANDLE fd, char *buf, int len)
    DWORD dwBytesRead;
    BOOL	rc;
 
+   FlushFileBuffers(fd);
    rc = ReadFile(fd, buf, len, &dwBytesRead, NULL);
    if (rc == FALSE)
       return(-1);
@@ -198,6 +201,7 @@ static int emulate_write(HANDLE fd, const char *buf, int len)
    BOOL	rc;
 
    rc = WriteFile(fd, buf, len, &dwBytesWritten, NULL);
+   FlushFileBuffers(fd);
    if (rc == FALSE)
       return(-1);
 
@@ -234,17 +238,11 @@ void script_init(const char *cparams)
    name=params;
    args=name;
    while ( *args && *args!=' ' ) ++args;
-   while ( *args && *args==' ' ) ++args;
+   while ( *args && *args==' ' ) *args++ = '\0';
    if ( *args==0 )
    {
       args=NULL;
    }
-#if 0
-   else
-   {
-      args[-1]=0;
-   }
-#endif
 
 #ifdef USE_PIPE
    /* Create two pipes */
@@ -293,17 +291,11 @@ void script_init(const char *cparams)
       /* Fill in argv[] */
       argv[0]=name;
       i=1;
-      while (args && *args)
+      while (args && *args && i < sizeof(argv)/sizeof(*argv)-1)
       {
-         argv[i]=args;
-         ++i;
+         argv[i++]=args;
          while ( *args && *args!=' ' ) ++args;
-         if ( *args )
-         {
-            *args=0;
-            ++args;
-         }
-         while ( *args && *args==' ' ) ++args;
+         while ( *args && *args==' ' ) *args++ = '\0';
       }
       argv[i]=NULL;
 
@@ -392,15 +384,11 @@ void script_init(const char *cparams)
    name=params;
    args=name;
    while ( *args && *args!=' ' ) ++args;
-   while ( *args && *args==' ' ) ++args;
+   while ( *args && *args==' ' ) *args++ = '\0';
    if ( *args==0 )
    {
       args=NULL;
    }
-   else
-   {
-      args[-1]=0;
-   }
 
    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES); 
    saAttr.bInheritHandle = TRUE;
@@ -463,7 +451,6 @@ void script_init(const char *cparams)
 	   return;
    }
 
-   CloseHandle(piProcInfo.hProcess);
    CloseHandle(piProcInfo.hThread);
 
    if (args)
@@ -492,6 +479,7 @@ void script_init(const char *cparams)
    scripts[num_scripts].watch=NULL;
    scripts[num_scripts].cmd_count=0;
    scripts[num_scripts].pid=piProcInfo.dwProcessId;
+   scripts[num_scripts].process = piProcInfo.hProcess;
    scripts[num_scripts].sync_watch = -1;
    ++num_scripts;
 
@@ -590,6 +578,8 @@ void script_process(fd_set *set)
    DWORD nAvailBytes = 0;
    char cTmp;
    BOOL bRC;
+   DWORD dwStatus;
+   BOOL bStatus;
 #endif
 
 
@@ -599,6 +589,8 @@ void script_process(fd_set *set)
 #ifndef WIN32
       if ( FD_ISSET(scripts[i].in_fd,set) )
 #else
+
+      bStatus = GetExitCodeProcess(scripts[i].process,&dwStatus);
       bRC = PeekNamedPipe(scripts[i].in_fd, &cTmp, 1, NULL, &nAvailBytes, NULL);
 	  if (nAvailBytes)
 #endif /* WIN32 */
@@ -634,20 +626,17 @@ void script_process(fd_set *set)
          return; /* Only process one script at a time */
       }
 #ifdef WIN32
-	  else if (!bRC) /* Error: assume dead */
+	  else if (!bRC || ( bStatus && ( dwStatus != STILL_ACTIVE ) ) ) /* Error: assume dead */
 		 script_dead(i);
 #endif /* WIN32 */
    }
 }
 
-void script_watch(char *cmd,char *data, int len, enum CmdFormat format)
+void script_watch(const char *cmd, char *data, int len, enum CmdFormat format)
 {
    int i;
    int w;
    int l;
-   int cl;
-
-   cl=strlen(cmd);
 
    /* For each script... */
    for (i=0;i<num_scripts;++i)
@@ -693,135 +682,137 @@ void script_watch(char *cmd,char *data, int len, enum CmdFormat format)
                   }
                   break;
                case STATS:
-                  /*
-                   * We cheat here and log each stat as a separate command, even
-                   * if the server sent a bunch of updates as a single message;
-                   * most scripts will be easier to write if they only parse a fixed
-                   * format.
-                   */
-                  while (len) {
-                     int be;
-                     int c; /* which stat */
+		  {
+                     /*
+                      * We cheat here and log each stat as a separate command, even
+                      * if the server sent a bunch of updates as a single message;
+                      * most scripts will be easier to write if they only parse a fixed
+                      * format.
+                      */
+                     int be = 0;
+                     while (len) {
+                        int c; /* which stat */
 
-                     be=sprintf(buf,"watch %s",cmd);
-                     c=*data;
-                     ++data; --len;
-                     if (c>=CS_STAT_RESIST_START && c<=CS_STAT_RESIST_END) {
-                        sprintf(buf+be," resists %d %d\n",c,GetShort_String(data));
-                        data+=2; len-=2;
-                     } else if (c >= CS_STAT_SKILLINFO && c < (CS_STAT_SKILLINFO+CS_NUM_SKILLS)) {
-                        sprintf(buf+be," skill %d %d %lld\n",c,*data,GetInt64_String(data+1));
-                        data+=9; len-=9;
-                     } else switch (c) {
-                        case CS_STAT_HP:
-                           sprintf(buf+be," hp %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_MAXHP:
-                           sprintf(buf+be," maxhp %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_SP:
-                           sprintf(buf+be," sp %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_MAXSP:
-                           sprintf(buf+be," maxspp %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_GRACE:
-                           sprintf(buf+be," grace %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_MAXGRACE:
-                           sprintf(buf+be," maxgrace %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_STR:
-                           sprintf(buf+be," str %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_INT:
-                           sprintf(buf+be," int %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_POW:
-                           sprintf(buf+be," pow %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_WIS:
-                           sprintf(buf+be," wis %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_DEX:
-                           sprintf(buf+be," dex %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_CON:
-                           sprintf(buf+be," con %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_CHA:
-                           sprintf(buf+be," cha %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_EXP:
-                           sprintf(buf+be," exp %d\n",GetInt_String(data));
-                           data+=4; len-=4; break;
-                        case CS_STAT_EXP64:
-                           sprintf(buf+be," exp %lld\n",GetInt64_String(data));
-                           data+=8; len-=8; break;
-                        case CS_STAT_LEVEL:
-                           sprintf(buf+be," level %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_WC:
-                           sprintf(buf+be," wc %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_AC:
-                           sprintf(buf+be," ac %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_DAM:
-                           sprintf(buf+be," dam %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_ARMOUR:
-                           sprintf(buf+be," armour %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_SPEED:
-                           sprintf(buf+be," speed %d\n",GetInt_String(data));
-                           data+=4; len-=4; break;
-                        case CS_STAT_FOOD:
-                           sprintf(buf+be," food %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_WEAP_SP:
-                           sprintf(buf+be," weap_sp %d\n",GetInt_String(data));
-                           data+=4; len-=4; break;
-                        case CS_STAT_FLAGS:
-                           sprintf(buf+be," flags %d\n",GetShort_String(data));
-                           data+=2; len-=2; break;
-                        case CS_STAT_WEIGHT_LIM:
-                           sprintf(buf+be," weight_lim %d\n",GetInt_String(data));
-                           data+=4; len-=4; break;
-                        case CS_STAT_SKILLEXP_AGILITY:
-                        case CS_STAT_SKILLEXP_PERSONAL:
-                        case CS_STAT_SKILLEXP_MENTAL:
-                        case CS_STAT_SKILLEXP_PHYSIQUE:
-                        case CS_STAT_SKILLEXP_MAGIC:
-                        case CS_STAT_SKILLEXP_WISDOM:
-                           sprintf(buf+be," skillexp %d %d\n",c,GetInt_String(data));
-                           data+=4; len-=4; break;
-                        case CS_STAT_SKILLEXP_AGLEVEL:
-                        case CS_STAT_SKILLEXP_PELEVEL:
-                        case CS_STAT_SKILLEXP_MELEVEL:
-                        case CS_STAT_SKILLEXP_PHLEVEL:
-                        case CS_STAT_SKILLEXP_MALEVEL:
-                        case CS_STAT_SKILLEXP_WILEVEL:
-                           sprintf(buf+be," skilllevel %d %d\n",c,GetShort_String(data));
-                           data+=2; len-=2; break;
+                        be+=sprintf(buf+be,"watch %s",cmd);
+                        c=*data;
+                        ++data; --len;
+                        if (c>=CS_STAT_RESIST_START && c<=CS_STAT_RESIST_END) {
+                           be+=sprintf(buf+be," resists %d %d\n",c,GetShort_String(data));
+                           data+=2; len-=2;
+                        } else if (c >= CS_STAT_SKILLINFO && c < (CS_STAT_SKILLINFO+CS_NUM_SKILLS)) {
+                           be+=sprintf(buf+be," skill %d %d %lld\n",c,*data,GetInt64_String(data+1));
+                           data+=9; len-=9;
+                        } else switch (c) {
+                           case CS_STAT_HP:
+                              be+=sprintf(buf+be," hp %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_MAXHP:
+                              be+=sprintf(buf+be," maxhp %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_SP:
+                              be+=sprintf(buf+be," sp %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_MAXSP:
+                              be+=sprintf(buf+be," maxspp %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_GRACE:
+                              be+=sprintf(buf+be," grace %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_MAXGRACE:
+                              be+=sprintf(buf+be," maxgrace %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_STR:
+                              be+=sprintf(buf+be," str %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_INT:
+                              be+=sprintf(buf+be," int %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_POW:
+                              be+=sprintf(buf+be," pow %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_WIS:
+                              be+=sprintf(buf+be," wis %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_DEX:
+                              be+=sprintf(buf+be," dex %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_CON:
+                              be+=sprintf(buf+be," con %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_CHA:
+                              be+=sprintf(buf+be," cha %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_EXP:
+                              be+=sprintf(buf+be," exp %d\n",GetInt_String(data));
+                              data+=4; len-=4; break;
+                           case CS_STAT_EXP64:
+                              be+=sprintf(buf+be," exp %lld\n",GetInt64_String(data));
+                              data+=8; len-=8; break;
+                           case CS_STAT_LEVEL:
+                              be+=sprintf(buf+be," level %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_WC:
+                              be+=sprintf(buf+be," wc %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_AC:
+                              be+=sprintf(buf+be," ac %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_DAM:
+                              be+=sprintf(buf+be," dam %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_ARMOUR:
+                              be+=sprintf(buf+be," armour %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_SPEED:
+                              be+=sprintf(buf+be," speed %d\n",GetInt_String(data));
+                              data+=4; len-=4; break;
+                           case CS_STAT_FOOD:
+                              be+=sprintf(buf+be," food %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_WEAP_SP:
+                              be+=sprintf(buf+be," weap_sp %d\n",GetInt_String(data));
+                              data+=4; len-=4; break;
+                           case CS_STAT_FLAGS:
+                              be+=sprintf(buf+be," flags %d\n",GetShort_String(data));
+                              data+=2; len-=2; break;
+                           case CS_STAT_WEIGHT_LIM:
+                              be+=sprintf(buf+be," weight_lim %d\n",GetInt_String(data));
+                              data+=4; len-=4; break;
+                           case CS_STAT_SKILLEXP_AGILITY:
+                           case CS_STAT_SKILLEXP_PERSONAL:
+                           case CS_STAT_SKILLEXP_MENTAL:
+                           case CS_STAT_SKILLEXP_PHYSIQUE:
+                           case CS_STAT_SKILLEXP_MAGIC:
+                           case CS_STAT_SKILLEXP_WISDOM:
+                              be+=sprintf(buf+be," skillexp %d %d\n",c,GetInt_String(data));
+                              data+=4; len-=4; break;
+                           case CS_STAT_SKILLEXP_AGLEVEL:
+                           case CS_STAT_SKILLEXP_PELEVEL:
+                           case CS_STAT_SKILLEXP_MELEVEL:
+                           case CS_STAT_SKILLEXP_PHLEVEL:
+                           case CS_STAT_SKILLEXP_MALEVEL:
+                           case CS_STAT_SKILLEXP_WILEVEL:
+                              be+=sprintf(buf+be," skilllevel %d %d\n",c,GetShort_String(data));
+                              data+=2; len-=2; break;
 
-                        case CS_STAT_RANGE: {
-                           int rlen=*data;
-                           ++data; --len;
-                           sprintf(buf+be," range %*.*s\n",rlen,rlen,data);
-                           data+=rlen; len-=rlen; break;
+                           case CS_STAT_RANGE: {
+                              int rlen=*data;
+                              ++data; --len;
+                              be+=sprintf(buf+be," range %*.*s\n",rlen,rlen,data);
+                              data+=rlen; len-=rlen; break;
+                           }
+                           case CS_STAT_TITLE: {
+                              int rlen=*data;
+                              ++data; --len;
+                              be+=sprintf(buf+be," title %*.*s\n",rlen,rlen,data);
+                              data+=rlen; len-=rlen; break;
+                           }
+                           default:
+                              be+=sprintf(buf+be," unknown %d %d bytes left\n",c,len);
+                              len=0;
                         }
-                        case CS_STAT_TITLE: {
-                           int rlen=*data;
-                           ++data; --len;
-                           sprintf(buf+be," title %*.*s\n",rlen,rlen,data);
-                           data+=rlen; len-=rlen; break;
-                        }
-                        default:
-                           sprintf(buf+be," unknown %d %d bytes left\n",c,len);
-                           len=0;
                      }
-                  }
+		  }
                   break;
                case MIXED:
                   /* magicmap */
@@ -911,7 +902,7 @@ void script_tell(const char *params)
    /* Send the message */
    write(scripts[i].out_fd,"scripttell ",11);
    write(scripts[i].out_fd,params,strlen(params));
-   write(scripts[i].out_fd,"\r\n",2);
+   write(scripts[i].out_fd,"\n",1);
 }
 
 static int script_by_name(const char *name)
@@ -953,6 +944,7 @@ static void script_dead(int i)
 #else
    CloseHandle(scripts[i].in_fd);
    CloseHandle(scripts[i].out_fd);
+   CloseHandle(scripts[i].process);
 #endif
    free(scripts[i].name);
    if ( scripts[i].params ) free(scripts[i].params);
@@ -1429,7 +1421,7 @@ static void script_process_cmd(int i)
  *   magic, cursed, damned, unpaid, locked, applied, open, was_open, inv_updated
  *    256     128     64      32       16      8       4      2         1
  */
-static void script_send_item(int i,char *head,item *it)
+static void script_send_item(int i, const char *head, const item *it)
 {
    char buf[4096];
    int flags;
