@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_script_h =
- *   "$Id: script.h,v 1.4 2005/02/10 07:03:36 mwedel Exp $";
+ *   "$Id: script.h,v 1.5 2006/02/23 20:18:28 akirschbaum Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -53,7 +53,7 @@ void script_killall(void);
 #endif
 void script_fdset(int *maxfd,fd_set *set);
 void script_process(fd_set *set);
-void script_watch(char *cmd,char *data, int len, enum CmdFormat format);
+void script_watch(const char *cmd, char *data, int len, enum CmdFormat format);
 void script_monitor(const char *command, int repeat, int must_send);
 void script_monitor_str(const char *command);
 void script_tell(const char *params);
