@@ -1,5 +1,5 @@
 char *rcsid_gtk2_image_c =
-    "$Id: image.c,v 1.3 2005/03/27 20:51:24 akirschbaum Exp $";
+    "$Id: image.c,v 1.7 2005/12/12 02:54:32 mwedel Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -55,6 +55,7 @@ char *rcsid_gtk2_image_c =
 #include "client.h"
 #include "image.h"
 #include "main.h"
+#include "mapdata.h"
 #include "gtk2proto.h"
 
 /* In main.c */
@@ -193,11 +194,14 @@ static void create_map_image(uint8 *data, PixmapInfo *pi)
 	}
 
 	for (i=0; i < pi->map_width * pi->map_height; i+= 4) {
-	    p = (uint32) (fog->pixels + i);
-	    g = ( ((p >> 24) & 0xff)  + ((p >> 16) & 0xff) + ((p >> 8) & 0xff)) / 3;
-	    p = (g << 24) | (g << 16) | (g << 8) | (p & 0xff);
-	    l = (uint32*) fog->pixels + i;
-	    *(uint32*) l = p;
+	    /* The pointer arithemtic below looks suspicious, but it is a patch that
+	     * is submitted, so just putting it in as submitted.
+	     * MSW 2004-05-11
+	     */
+	    p = (uint32*) (fog->pixels + i);
+	    g = ( ((*p >> 24) & 0xff)  + ((*p >> 16) & 0xff) + ((*p >> 8) & 0xff)) / 3;
+            l = (uint32*) fog->pixels + i;
+	    *(uint32*) l = (g << 24) | (g << 16) | (g << 8) | (*p & 0xff);
 	}
 
 	SDL_UnlockSurface(fog);
@@ -230,7 +234,16 @@ static void free_pixmap(PixmapInfo *pi)
 	    SDL_FreeSurface(pi->map_image);
 	    free(((SDL_Surface*)pi->map_image)->pixels);
 	    SDL_FreeSurface(pi->fog_image);
-	    free(((SDL_Surface*)pi->fog_image)->pixels);
+	    /* Minor memory leak here - SDL_FreeSurface() frees the pixel
+	     * data _unless_ SDL_CreateRGBSurfaceFrom() was used to create
+	     * the surface.  SDL_CreateRGBSurfaceFrom() is used to create the
+	     * map data, which is why we need the free there.  The reason this
+	     * is a minor memory look is because SDL_CreateRGBSurfaceFrom() is
+	     * used to create the question mark image, and without this
+	     * free, that data is not freed.  However, with this, client
+	     * crashes after disconnecting from server with double free.
+	     */
+/*	    free(((SDL_Surface*)pi->fog_image)->pixels);*/
 	}
 #endif
 	}
@@ -394,8 +407,6 @@ void reset_image_data()
 	pixmaps[i] = pixmaps[0];
     }
     }
-    memset( the_map.cells[0], 0, sizeof( sizeof( struct MapCell)*
-					 the_map.x * the_map.y ));
 }
 
 
@@ -500,12 +511,16 @@ void init_cache_data()
 	pixmaps[0]->map_image = SDL_CreateRGBSurfaceFrom(question_sdl,
 		32, 32, 1, 4, 1, 1, 1, 1);
 	SDL_SetAlpha(pixmaps[0]->map_image, SDL_SRCALPHA, 70);
+	pixmaps[0]->fog_image = SDL_CreateRGBSurfaceFrom(question_sdl,
+		32, 32, 1, 4, 1, 1, 1, 1);
+	SDL_SetAlpha(pixmaps[0]->fog_image, SDL_SRCALPHA, 70);
     }
     else
 #endif
     if (use_config[CONFIG_DISPLAYMODE]==CFG_DM_PIXMAP)
     {
 	pixmaps[0]->map_image =  pixmaps[0]->icon_image;
+	pixmaps[0]->fog_image =  pixmaps[0]->icon_image;
 	pixmaps[0]->map_mask =  pixmaps[0]->icon_mask;
     }
 #ifdef HAVE_OPENGL
@@ -515,6 +530,7 @@ void init_cache_data()
 #endif
 
     pixmaps[0]->icon_width = pixmaps[0]->icon_height = pixmaps[0]->map_width = pixmaps[0]->map_height = map_image_size;
+    pixmaps[0]->smooth_face = 0;
 
     /* Don't do anything special for SDL image - rather, that drawing
      * code will check to see if there is no data
