@@ -1,5 +1,5 @@
 char *rcsid_gtk2_inventory_c =
-    "$Id: inventory.c,v 1.6 2005/06/30 06:37:30 mwedel Exp $";
+    "$Id: inventory.c,v 1.10 2006/03/11 08:42:03 mwedel Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -81,8 +81,8 @@ enum {
 static int num_inv_notebook_pages=0;
 
 typedef struct {
-    char    *name;		/* Name of this page, for use with the show command */
-    char    **xpm;		/* Icon to draw for the notebook selector */
+    const char *name;		/* Name of this page, for use with the show command */
+    const char *const *xpm;	/* Icon to draw for the notebook selector */
     int(*show_func) (item *it);	/* Function that takes an item and */
 				/* returns INV_SHOW_* above on whether to show this */
 				/* item and if it should be shown in color */
@@ -118,7 +118,7 @@ Notebook_Info	inv_notebooks[NUM_INV_LISTS] = {
 
 
 enum {
-LIST_NONE, LIST_ICON, LIST_NAME, LIST_WEIGHT, LIST_OBJECT, LIST_BACKGROUND, LIST_NUM_COLUMNS
+LIST_NONE, LIST_ICON, LIST_NAME, LIST_WEIGHT, LIST_OBJECT, LIST_BACKGROUND, LIST_TYPE, LIST_NUM_COLUMNS
 };
 
 
@@ -166,7 +166,8 @@ static void list_item_action(GdkEventButton *event, item *tmp)
     }
     else if (event->button == 3) {
 	if (tmp->locked) {
-	    draw_info ("This item is locked.",NDI_BLACK);
+	    draw_info ("This item is locked. To drop it, first unlock by shift+leftclicking on it.",
+		NDI_BLACK);
 	} else {
 	    uint32	dest;
 
@@ -299,6 +300,7 @@ static void setup_list_columns(GtkWidget *treeview)
 
     gtk_tree_view_column_set_min_width(column, image_size);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+    gtk_tree_view_column_set_sort_column_id(column, LIST_TYPE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
     renderer = gtk_cell_renderer_text_new ();
@@ -306,6 +308,7 @@ static void setup_list_columns(GtkWidget *treeview)
                                                       "text", LIST_NAME,
                                                       NULL);
     gtk_tree_view_column_set_expand(column, TRUE);
+    gtk_tree_view_column_set_sort_column_id(column, LIST_NAME);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
     gtk_tree_view_column_add_attribute(column, renderer, "background-gdk", LIST_BACKGROUND);
     gtk_tree_view_set_expander_column(GTK_TREE_VIEW (treeview), column);
@@ -323,6 +326,7 @@ static void setup_list_columns(GtkWidget *treeview)
     gtk_tree_view_column_set_min_width(column, 50);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
 
+    gtk_tree_view_column_set_sort_column_id(column, LIST_WEIGHT);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
     gtk_tree_view_column_add_attribute(column, renderer, "background-gdk", LIST_BACKGROUND);
 
@@ -358,7 +362,8 @@ void inventory_init(GtkWidget *window_root)
 				G_TYPE_STRING,
 				G_TYPE_STRING,
 				G_TYPE_POINTER,
-				GDK_TYPE_COLOR);
+				GDK_TYPE_COLOR,
+				G_TYPE_INT);
 
     gtk_tree_view_set_model(GTK_TREE_VIEW(treeview_look), GTK_TREE_MODEL(store_look));
     setup_list_columns(treeview_look);
@@ -392,7 +397,8 @@ void inventory_init(GtkWidget *window_root)
 				G_TYPE_STRING,
 				G_TYPE_STRING,
 				G_TYPE_POINTER,
-				GDK_TYPE_COLOR);
+				GDK_TYPE_COLOR,
+				G_TYPE_INT);
 	    inv_notebooks[i].treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(
 						    inv_notebooks[i].treestore));
 
@@ -489,18 +495,13 @@ void set_weight_limit (uint32 wlim)
 void get_row_color(item *it, int *fg, int *bg)
 {
     *fg = NDI_BLACK;
-    if (!use_config[CONFIG_COLORINV]) {
-	*bg = NDI_WHITE;
-	*fg = NDI_BLACK;
-    } else {
-	if (it->cursed || it->damned) {
-	    if (it->magical) *bg = NDI_NAVY;
-	    else *bg = NDI_RED;
-	} else if (it->magical) {
-	    *bg = NDI_BLUE;
-	}
-	else *bg = NDI_WHITE;
+    if (it->cursed || it->damned) {
+	if (it->magical) *bg = NDI_NAVY;
+	else *bg = NDI_RED;
+    } else if (it->magical) {
+	*bg = NDI_BLUE;
     }
+    else *bg = NDI_WHITE;
 }
 
 /***************************************************************************
@@ -563,6 +564,7 @@ static void add_object_to_store(item *it, GtkTreeStore *store,
 		LIST_WEIGHT, buf,
 		LIST_BACKGROUND, &root_color[bg],
 		LIST_OBJECT, it,
+		LIST_TYPE, it->type,
                 -1);
 }
 
