@@ -1,5 +1,5 @@
 char *rcsid_gtk2_menubar_c =
-    "$Id: menubar.c,v 1.1 2005/03/02 07:52:04 mwedel Exp $";
+    "$Id: menubar.c,v 1.2 2006/02/20 07:22:04 mwedel Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -44,6 +44,27 @@ char *rcsid_gtk2_menubar_c =
  *    So I don't use the stock item.
  */
 
+void
+on_disconnect_activate                 (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    extern gint csocket_fd;
+
+#ifdef WIN32
+    closesocket(csocket.fd);
+#else
+    close(csocket.fd);
+#endif
+    csocket.fd = -1;
+    if (csocket_fd) {
+        gdk_input_remove(csocket_fd);
+        csocket_fd=0;
+        gtk_main_quit();
+    }
+}
+
+
+
 void
 menu_quit_program                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
