@@ -1,5 +1,5 @@
 char *rcsid_gtk_opengl_c =
-    "$Id: opengl.c,v 1.2 2005/03/23 22:19:02 tchize Exp $";
+    "$Id: opengl.c,v 1.5 2006/05/15 05:57:43 mwedel Exp $";
 
 /*
     Crossfire client, a client program for the crossfire program.
@@ -48,6 +48,7 @@ char *rcsid_gtk_opengl_c =
 #include "main.h"
 #include "image.h"
 #include <client.h>
+#include "mapdata.h"
 
 #include "gtk2proto.h"
 
@@ -117,13 +118,36 @@ void init_opengl(GtkWidget* drawingarea)
 	GLX_ALPHA_SIZE, 4, 
 	GLX_DEPTH_SIZE, 16,
 	None };
+    XSetWindowAttributes attr;
+    Visual *v;
 
     /* Need to tuck these away, because they are needed for glXSwappBuffers() */
     display = GDK_WINDOW_XDISPLAY(drawingarea->window);
-    window = GDK_WINDOW_XID(drawingarea->window);
-
     vi = glXChooseVisual(display,
 			 gdk_x11_get_default_screen (), attrListDbl);
+
+    width = drawingarea->allocation.width;
+    height = drawingarea->allocation.height;
+
+    /* On many systems, the default visual used by the display doesn't
+     * have the features we need (alpha channel, etc).  The only way
+     * around this is to create a subwindow with the ideal
+     * visual.  As an oddity, we need to create a colormap
+     * if using a different visual or we get a BadMatch error.
+     */
+    v = DefaultVisual(display, gdk_x11_get_default_screen ());
+    attr.colormap= XCreateColormap(display, GDK_WINDOW_XID(drawingarea->window),
+				   vi->visual, AllocNone);
+
+    window = XCreateWindow(display, GDK_WINDOW_XID(drawingarea->window),
+		0, 0, width, height, 0, 
+			   vi->depth,
+			InputOutput, 
+			   vi->visual,
+			   CWColormap, &attr);
+
+    XMapWindow(display,window);
+
     if (!vi) {
 	LOG(LOG_WARNING,"gtk::init_opengl", "Could not get double buffered screen!\n");
     }
@@ -139,8 +163,6 @@ void init_opengl(GtkWidget* drawingarea)
     else
 	LOG(LOG_INFO,"gtk::init_opengl", "Direct rendering is not available!\n");
 
-    width = drawingarea->allocation.width;
-    height = drawingarea->allocation.height;
     init_opengl_common();
 }
 
@@ -315,7 +337,7 @@ static void drawsmooth_opengl (int x, int y, int mx, int my, int layer) {
             slevels[i]=0;
             sfaces[i]=0; /*black picture*/
         }
-        if (the_map.cells[emx][emy].smooth[layer]<=the_map.cells[mx][my].smooth[layer] ||
+        else if (the_map.cells[emx][emy].smooth[layer]<=the_map.cells[mx][my].smooth[layer] ||
 	    the_map.cells[emx][emy].heads[layer].face == 0){
             slevels[i]=0;
             sfaces[i]=0; /*black picture*/
@@ -454,7 +476,7 @@ static void draw_smoothing(int layer)
 void opengl_gen_map(int redraw) {
     long elapsed1, elapsed2;
     struct timeval tv1, tv2,tv3;
-    int mx,my, layer,x,y, d1, d2, d3, num_dark, got_smooth;
+    int mx,my, layer,x,y, d1, d2, d3, num_dark, got_smooth, face, t1, t2;
 
     if (time_map_redraw)
 	gettimeofday(&tv1, NULL);
@@ -629,6 +651,44 @@ void opengl_gen_map(int redraw) {
 
 			    got_smooth=1;
 			}
+		    }
+		    if ((face=mapdata_bigface_head(x, y, layer, &t1, &t2))!=0) {
+			if (pixmaps[face]->map_texture) {
+			    int nx, ny;
+
+			    /* nx, ny are the location of the top/left side of the image to draw. */
+
+			    nx = (x+1) * map_image_size - pixmaps[face]->map_width;
+			    ny = (y+1) * map_image_size - pixmaps[face]->map_height;
+
+			    /* if both nx and ny are outside visible area, don't need to do anything more */
+			    if (nx > width && ny > height) continue;
+
+			    /* There are some issues with this - it is really the head of the
+			     * object that is determining fog of war logic.  I don't have good solution
+			     * to that, other than to live with it.
+			     */
+			    if (the_map.cells[mx][my].cleared)
+				glBindTexture(GL_TEXTURE_2D, pixmaps[face]->fog_texture);
+			    else
+				glBindTexture(GL_TEXTURE_2D, pixmaps[face]->map_texture);
+
+			    glBegin(GL_QUADS);
+
+			    glTexCoord2f(0.0f, 0.0f);
+			    glVertex3i(nx, ny, 0);
+
+			    glTexCoord2f(1.0f, 0.0f); 
+			    glVertex3i( (x+1) * map_image_size, ny, 0);
+
+			    glTexCoord2f(1.0f, 1.0f);
+			    glVertex3i( (x+1) * map_image_size, (y+1) * map_image_size, 0);
+
+			    glTexCoord2f(0.0f, 1.0f);
+			    glVertex3i(nx, (y+1) * map_image_size, 0);
+
+			    glEnd();
+			}
 		    } /* If this space has a valid face */
 		} /* If last layer/else not last layer */
 	    } /* for x loop */
@@ -828,7 +888,13 @@ void create_opengl_question_mark()
     glTexImage2D(GL_TEXTURE_2D, 0, 4, question_width, question_height, 
                0, GL_RGBA, GL_UNSIGNED_BYTE, &question[0][0][0]);
 
-    pixmaps[0]->fog_texture=0;
+    glGenTextures(1, &pixmaps[0]->fog_texture);
+    glBindTexture(GL_TEXTURE_2D, pixmaps[0]->fog_texture);
+    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+
+    glTexImage2D(GL_TEXTURE_2D, 0, 4, question_width, question_height,
+               0, GL_RGBA, GL_UNSIGNED_BYTE, &question[0][0][0]);
 }
 
 #endif
