@@ -1,5 +1,5 @@
 char *rcsid_gtk_pickup_c =
-    "$Id: pickup.c,v 1.1 2005/04/17 21:10:14 mwedel Exp $";
+    "$Id: pickup.c,v 1.2 2005/11/01 13:46:10 akirschbaum Exp $";
 
 /*
     Crossfire client, a client program for the crossfire program.
@@ -80,6 +80,8 @@ char *rcsid_gtk_pickup_c =
 #define PU_READABLES		0x00400000
 #define PU_MAGIC_DEVICE		0x00800000
 
+#define PU_NOT_CURSED		0x01000000
+
 static unsigned int pmode=0;
 
 /* This covers the pickup operations.  Unfortunately, it isn't easy (possible?)
@@ -257,6 +259,14 @@ on_menu_wands_rods_horns_activate      (GtkMenuItem     *menuitem,
 
 }
 
+void
+on_menu_not_cursed_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    new_menu_pickup(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)), PU_NOT_CURSED);
+
+}
+
 /***************************************************************************
  * Weapons submenu
  ***************************************************************************/
