@@ -1,5 +1,5 @@
-char *rcsid_gtk_config_c =
-    "$Id: config.c,v 1.21 2005/05/30 21:01:57 ryo_saeba Exp $";
+const char *rcsid_gtk_config_c =
+    "$Id: config.c,v 1.31 2006/02/28 08:09:52 mwedel Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -92,11 +92,11 @@ char *rcsid_gtk_config_c =
  * 2) Since the radio button is several widgets, its not as simple
  * as normal buttons to map them to a config value.  Instead,
  * use a range so that it is easy to tell what config value
- * your button belongs to, ie, 100-199 is for the lighting
- * options, 200-299 is for the resistance options, etc.
+ * your button belongs to, eg, 100-199 is for the lighting
+ * options.
  */
 
-#define MAX_BUTTONS	    36
+#define MAX_BUTTONS	    33
 #define RBUTTON	    1
 #define CBUTTON	    2
 #define SEPERATOR   3	    /* Seperator in the window */
@@ -114,7 +114,7 @@ typedef struct {
     int		type;
     int		config;
     int		flags;
-    char	*label;
+    const char	*label;
 } CButtons;
 
 static GtkWidget *gtkwin_config = NULL,	    /* main window */
@@ -125,13 +125,11 @@ static GtkWidget *gtkwin_config = NULL,	    /* main window */
  * config gui elements.
  */
 
-CButtons cbuttons[MAX_BUTTONS] = {
+static CButtons cbuttons[MAX_BUTTONS] = {
 {NULL, 	    CBUTTON,	    CONFIG_FOODBEEP,	FLAG_UPDATE,
     "Beep When Food is Low"},
-{NULL, 	    CBUTTON,	    CONFIG_COLORINV,	0,
-    "Colored Inventory Lists"},
-{NULL, 	    CBUTTON,	    CONFIG_COLORTXT,	FLAG_UPDATE,
-    "Colored Information Text"},
+{NULL, 	    CBUTTON,	    CONFIG_TIMESTAMP,	FLAG_UPDATE,
+    "Timestamp Messages"},
 {NULL, 	    SPIN_CWINDOW,   CONFIG_CWINDOW,	FLAG_UPDATE,
     "Command Window"},
 {NULL, 	    CBUTTON,	    CONFIG_ECHO,	FLAG_UPDATE,
@@ -142,6 +140,8 @@ CButtons cbuttons[MAX_BUTTONS] = {
     "Gradually change stat bar color based on value of the stat.\nThis option will result in some extra CPU usage."},
 {NULL, 	    CBUTTON,	    CONFIG_POPUPS,	FLAG_UPDATE,
     "Popup Windows"},
+{NULL, 	    CBUTTON,	    CONFIG_SIGNPOPUP,	FLAG_UPDATE,
+    "Popup Sign Windows (need Popup Windows checked to be used)"},
 {NULL, 	    CBUTTON,	    CONFIG_SPLASH,	FLAG_UPDATE,
     "Splash Window"},
 {NULL, 	    CBUTTON,	    CONFIG_SHOWICON,	FLAG_UPDATE,
@@ -159,16 +159,10 @@ CButtons cbuttons[MAX_BUTTONS] = {
     "improves performance but bugs in\n gtk make the client unstable if this is used." /**/
     "This may work better with gtk 2.0"},
 {NULL, 	    CBUTTON,	    CONFIG_APPLY_CONTAINER,	FLAG_UPDATE,
-    "Automatically re-applies a container when you use apply to close it.  If off, when you use apply to close the container, it stays unapplied"},
+    "Automatically re-applies a container when you use apply to close it. \nIf off, when you use apply to close the container, it stays unapplied"},
 
-{NULL,	    SEPERATOR,		0,		0,
-    "Options on how to display resistances:"},
-{NULL, 	    RBUTTON,	    200,		0,
-    "Don't use a scrollbar, will show a maximum of 7 resistances."},
-{NULL, 	    RBUTTON,	    201,		0,
-    "Display all resistances in a single column, will use a single scrollbar."},
-{NULL, 	    RBUTTON,	    202,		0,
-    "Display all resistances in the form of a double column, may result in one\nor two scrollbars being created"},
+{NULL, 	    CBUTTON,	    CONFIG_RESISTS,	0,
+    "Display resistances in two columns rather than only one."},
 
 /* The following items are shown in the map tag.
  * I grouped them together to make reading them a bit easier,
@@ -220,9 +214,18 @@ static void set_config_value(int cval, int value)
 	use_config[cbuttons[cval].config] = value;
 }
 
+static int splitwin_toggling = FALSE;
+ 
+void main_window_destroyed() {
+    if (!splitwin_toggling) {
+        client_exit();
+    }
+}
 
 static void toggle_splitwin(int newval)
 {
+    splitwin_toggling = TRUE;
+
     inventory_splitwin_toggling();
 	gtk_widget_destroy(gtkwin_root);
         
@@ -238,10 +241,12 @@ static void toggle_splitwin(int newval)
     }
 
 	create_windows();
-	display_map_doneupdate(TRUE);
+	display_map_doneupdate(TRUE, FALSE);
 	draw_stats (1);
     update_list_labels(&inv_list); /* After exploding or unexploding client, redraw weight labels. */
     update_list_labels(&look_list);
+
+    splitwin_toggling = FALSE;
 }
 
 /* Ok, here it sets the config and saves it. This is sorta dangerous, and I'm not sure
@@ -251,10 +256,10 @@ static void toggle_splitwin(int newval)
 
 #define IS_DIFFERENT(TYPE) (want_config[TYPE] != use_config[TYPE])
 
-void applyconfig () {
+static void applyconfig(void) {
 
     int onbutton;
-    int lighting = 0, resistances=0;
+    int lighting = 0;
 
     if (face_info.want_faceset) free(face_info.want_faceset);
     face_info.want_faceset = strdup_local(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(faceset_combo)->entry)));
@@ -277,8 +282,6 @@ void applyconfig () {
 	    if ( GTK_TOGGLE_BUTTON (cbuttons[onbutton].widget)->active) {
 		if ( cbuttons[onbutton].config >= 100 &&  cbuttons[onbutton].config < 200)
 		    lighting = cbuttons[onbutton].config - 100;
-		else if ( cbuttons[onbutton].config >= 200 &&  cbuttons[onbutton].config < 300)
-		    resistances = cbuttons[onbutton].config - 200;
 	    }
 	}
     } /* for onbutton ... loop */
@@ -301,10 +304,6 @@ void applyconfig () {
 	}
 	use_config[CONFIG_SOUND] = want_config[CONFIG_SOUND];
     }
-    if (IS_DIFFERENT(CONFIG_COLORINV)) {
-	use_config[CONFIG_COLORINV] = want_config[CONFIG_COLORINV];
-	inventory_update_colorinv();
-    }
     if (IS_DIFFERENT(CONFIG_TOOLTIPS)) {
 	if (want_config[CONFIG_TOOLTIPS]) gtk_tooltips_enable(tooltips);
 	else gtk_tooltips_disable(tooltips);
@@ -333,6 +332,7 @@ void applyconfig () {
     }
     if (IS_DIFFERENT(CONFIG_RESISTS)) {    
 	use_config[CONFIG_RESISTS] = want_config[CONFIG_RESISTS];
+	resize_resistance_table(use_config[CONFIG_RESISTS]);
     }
     if (!use_config[CONFIG_GRAD_COLOR]) {
 	reset_stat_bars();
@@ -351,7 +351,6 @@ void applyconfig () {
 	if( csocket.fd)
 	    cs_print_string(csocket.fd, "mapredraw");
     }
-    want_config[CONFIG_RESISTS] = resistances;
     if (want_config[CONFIG_RESISTS] != use_config[CONFIG_RESISTS]) {
 	resize_resistance_table(want_config[CONFIG_RESISTS]);
 	use_config[CONFIG_RESISTS] = want_config[CONFIG_RESISTS];
@@ -364,7 +363,7 @@ void applyconfig () {
  * if it's actually possible to do dynamic reconfiguration of everything this way.
  */
 
-void saveconfig () {
+static void saveconfig(void) {
 
     /* No idea why applyconfig was basically replicated - just call the
      * function instead!
@@ -399,7 +398,7 @@ void configdialog(GtkWidget *widget) {
     GList	*flist;
     int i, num_extras=0;
 
-    gchar *titles[] ={"#","Key","(#)","Mods","Command"};	   
+    const gchar *titles[] ={"#","Key","(#)","Mods","Command"};	   
 
     /* If the window isnt already up (in which case it's just raised) */
     if(!gtkwin_config) {
@@ -473,8 +472,6 @@ void configdialog(GtkWidget *widget) {
 		}
 		if ((want_config[CONFIG_LIGHTING]+100) == cbuttons[i].config)
 		    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cbuttons[i].widget), 1);
-		if ((want_config[CONFIG_RESISTS]+200) == cbuttons[i].config)
-		    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cbuttons[i].widget), 1);
 	    }
 	    else if (cbuttons[i].type & SPIN) {
 		GtkAdjustment *adj=NULL;
@@ -712,7 +709,7 @@ void configdialog(GtkWidget *widget) {
 }
 
 
-void load_defaults()
+void load_defaults(void)
 {
     char path[MAX_BUF],inbuf[MAX_BUF],*cp;
     FILE *fp;
@@ -786,16 +783,15 @@ void load_defaults()
 	    face_info.want_faceset = strdup_local(cp);	/* memory leak ! */
 	    continue;
 	}
-	/* legacy, as this is now just saved as 'lighting' */
-	else if (!strcmp(inbuf, "per_tile_lighting")) {
-	    if (val) want_config[CONFIG_LIGHTING] = CFG_LT_TILE;
-	}
-	else if (!strcmp(inbuf, "per_pixel_lighting")) {
-	    if (val) want_config[CONFIG_LIGHTING] = CFG_LT_PIXEL;
-	}
+	/* legacy support for the old resistances values, we need to adjust the values to the new form */
 	else if (!strcmp(inbuf, "resists")) {
-	    if (val) want_config[CONFIG_RESISTS] = val;
+	    if (val) want_config[CONFIG_RESISTS] = val-1;
 	}
+        else if (!strcmp(inbuf, "sdl")) {
+            if (val) want_config[CONFIG_DISPLAYMODE] = CFG_DM_SDL;
+	}
+
+	
 	else LOG(LOG_WARNING,"gtk::load_defaults","Unknown line in gdefaults: %s %s", inbuf, cp);
     }
     fclose(fp);
@@ -819,12 +815,6 @@ void load_defaults()
 	LOG(LOG_WARNING,"gtk::load_defaults","No lighting mechanism selected - will not use darkness code");
 	want_config[CONFIG_DARKNESS] = FALSE;
     }
-    if (want_config[CONFIG_RESISTS] > 2) {
-	LOG(LOG_WARNING,"gtk::load_defaults","ignoring resists display value read for gdafaults file.\n"
-            "Invalid value (%d), must be one value of 0,1 or 2.",
-            want_config[CONFIG_RESISTS]);
-	want_config[CONFIG_RESISTS] = 0;
-    }
     
     /* Make sure the map size os OK */
     if (want_config[CONFIG_MAPWIDTH] < 9 || want_config[CONFIG_MAPWIDTH] > MAP_MAX_SIZE) {
@@ -840,6 +830,11 @@ void load_defaults()
 	want_config[CONFIG_MAPHEIGHT] = use_config[CONFIG_MAPHEIGHT];
     }
 
+#ifndef HAVE_SDL
+	/* If SDL is not built in, having SDL mode turned on causes many issues. */
+    want_config[CONFIG_DISPLAYMODE] = CFG_DM_PIXMAP;
+#endif
+
     /* Now copy over the values just loaded */
     for (i=0; i<CONFIG_NUMS; i++) {
 	use_config[i] = want_config[i];
@@ -852,7 +847,7 @@ void load_defaults()
 
 }
 
-void save_defaults()
+void save_defaults(void)
 {
     char path[MAX_BUF],buf[MAX_BUF];
     FILE *fp;
