@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_gx11_h =
- *   "$Id: gx11.h,v 1.13 2005/06/01 06:19:14 akirschbaum Exp $";
+ *   "$Id: gx11.h,v 1.15 2005/09/04 20:40:05 ryo_saeba Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -40,7 +40,6 @@
 #include "item.h"
 
 extern int map_size, image_size, map_image_size, map_image_half_size;
-extern uint8 map_did_scroll;
 extern uint8 updatekeycodes, time_map_redraw,redraw_needed;
 extern int updatelock;
 
@@ -86,6 +85,9 @@ extern GtkTooltips  *tooltips;
 typedef struct {
   item *env;		  /* Environment shown in window */
   char title[MAX_BUF];  /* title of item list */
+  char last_title[MAX_BUF];
+  char last_weight[MAX_BUF];
+  char last_maxweight[MAX_BUF];
   
   GtkWidget *label; /* e.g. "Inventory:", "keyring:" */
   GtkWidget *weightlabel;
@@ -99,9 +101,6 @@ typedef struct {
 
 extern itemlist inv_list, look_list;
 
-/* This was just a test I put in to try different redraw methods */
-#define ALTERNATE_MAP_REDRAW	0
-
 typedef struct news_entry{
     char* title;
     char* content;
