@@ -58,7 +58,7 @@ static inventory_viewer * new_inventory_viewer(item * container, itemfilter filt
     GtkWidget * list;
     GtkWidget * scroll_window;
     GtkStyle * liststyle;
-    gchar *titles[] = {"?", "Name", "Weight"};
+    const gchar *titles[] = {"?", "Name", "Weight"};
         
     scroll_window = gtk_scrolled_window_new (0,0);
 
@@ -289,7 +289,7 @@ static void animate_one_item(gpointer item_x, gpointer ignored) {
     g_list_foreach(views, animate_item, it);
 }
 
-static void animate_items() {
+static void animate_items(void) {
     g_list_foreach(animated_items, animate_one_item, NULL);    
 }
 
@@ -459,7 +459,7 @@ static void item_tick_per_view(gpointer data, gpointer user_data_ignored) {
     } 
 }
 
-static void itemview_tick() {
+static void itemview_tick(void) {
     animate_items(); 
     
     g_list_foreach(views, item_tick_per_view, NULL);
@@ -665,7 +665,8 @@ static void list_button_event(
     }
     if (event->button==3) {
         if (it->locked) {
-            draw_info ("This item is locked.",NDI_BLACK);
+            draw_info ("This item is locked. To drop it, first unlock by shift+leftclicking on it.",
+		NDI_BLACK);
         } else {
             cpl.count = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(counttext));
             client_send_move (view->move_dest->tag, it->tag, cpl.count);
@@ -717,8 +718,8 @@ static bool show_locked(item * it) { return it->locked; }
 static bool show_unlocked(item * it) { return !(it->locked); }
 
 typedef struct {
-  char * name;
-  char ** xpm;
+  const char *name;
+  const char *const *xpm;
   itemfilter filter;
   bool highlight;
 } fixed_tab_init;
@@ -805,6 +806,12 @@ static void resize_left_widget_one(gpointer view_x, gpointer total_width_x) {
     inventory_viewer  * view = (inventory_viewer *)view_x;
     gint total_width = GPOINTER_TO_INT(total_width_x);
     
+    if (view == NULL) {
+        /* Weird. This is never set as a signal handler, but somehow gtk1.2
+           decides to call this with a NULL view when going to split windows.
+        */
+        return;
+    }
     
     gtk_clist_set_column_width(GTK_CLIST(view->list),
         1, total_width - view->image_width);
@@ -823,9 +830,18 @@ hand, I don't know any alternative, aside from using clist's automatic sizing fe
 
 static void resize_left_widgets(GtkWidget *widget, GtkAllocation *event) {
     static gint old_total_width = 0;
+    inventory_viewer * hack; 
+    gint total_width; 
+    
+    /* If GTK can unexpectedly call resize_left_widget_one() as a signal
+       handler when the inventory widgets temporarily don't exist, then we
+       might as well watch our tail here, too. */
+    if (inv_viewers == NULL) return;
+    if (look_viewer == NULL) return;
+    
     /* HACK Extract the first inventory-viewer. */
-    inventory_viewer * hack = (inventory_viewer *)(inv_viewers->data);
-    gint total_width = GTK_CLIST(hack->list)->clist_window_width - MAGIC_SAFE_WIDTH; 
+    hack = (inventory_viewer *)(inv_viewers->data);
+    total_width = GTK_CLIST(hack->list)->clist_window_width - MAGIC_SAFE_WIDTH; 
     
     if (old_total_width == total_width) return;
     old_total_width = total_width;
@@ -916,10 +932,8 @@ static GtkWidget * get_inv_widget(void) {
         view = new_inventory_viewer(cpl.ob, i->filter, cpl.below); /* player to ground */
         view->face_column_resizes = FALSE;
                 
-        if (use_config[CONFIG_COLORINV]) {
-            inventory_viewer_set_highlight(view, i->highlight);
-            highlit_inv_viewers = g_list_append(highlit_inv_viewers, view);
-        }
+        inventory_viewer_set_highlight(view, i->highlight);
+        highlit_inv_viewers = g_list_append(highlit_inv_viewers, view);
         
         inv_viewers = g_list_append(inv_viewers, view);
 
@@ -933,7 +947,7 @@ static GtkWidget * get_inv_widget(void) {
         
         gtk_signal_connect(GTK_OBJECT(view->list), 
             "size-allocate",
-            resize_left_widgets,
+            (GtkSignalFunc)resize_left_widgets,
             NULL);
 
         /* Since the program will automatically adjust these, any changes
@@ -949,7 +963,7 @@ static GtkWidget * get_inv_widget(void) {
         redraws dirty tabs when they *become* visible. */
         gtk_signal_connect(GTK_OBJECT(view->list),
             "visibility-notify-event",
-            redraw_on_show,
+            (GtkSignalFunc)redraw_on_show,
             view);
         
          gtk_widget_add_events(view->list, GDK_VISIBILITY_NOTIFY_MASK);        
@@ -962,7 +976,7 @@ static GtkWidget * get_inv_widget(void) {
 
     
 
-static GtkWidget * get_look_widget() {
+static GtkWidget *get_look_widget(void) {
     if (look_widget != NULL) {
         return look_widget;
     }
@@ -983,6 +997,8 @@ static GtkWidget * get_look_widget() {
 
 itemlist look_list, inv_list;
 
+GtkWidget *closebutton;
+
 /* forward */
 static void close_container_callback(item *op);
 
@@ -991,10 +1007,12 @@ void get_look_display(GtkWidget *frame)
 {
   GtkWidget *vbox1;
   GtkWidget *hbox1;
-  GtkWidget *closebutton;
   
   look_list.env = cpl.below;
   strcpy (look_list.title, "You see:");
+  strcpy (look_list.last_title, look_list.title);
+  strcpy (look_list.last_weight, "0");
+  strcpy (look_list.last_maxweight, "0");
   look_list.show_weight = TRUE;
   look_list.weight_limit = 0;
     
@@ -1010,6 +1028,7 @@ void get_look_display(GtkWidget *frame)
   gtk_signal_connect_object (GTK_OBJECT (closebutton), "clicked",
 			       GTK_SIGNAL_FUNC(close_container_callback),
 			       NULL);
+  gtk_widget_set_sensitive(closebutton, FALSE);
   gtk_box_pack_start (GTK_BOX(hbox1),closebutton, FALSE, FALSE, 2);
   gtk_widget_show (closebutton);
   gtk_tooltips_set_tip (tooltips, closebutton, 
@@ -1055,6 +1074,9 @@ void get_inv_display(GtkWidget *frame)
   GtkAdjustment *adj;
 
   strcpy (inv_list.title, "Inventory:");
+  strcpy (inv_list.last_title, inv_list.title);
+  strcpy (inv_list.last_weight, "0");
+  strcpy (inv_list.last_maxweight, "0");
   inv_list.env = cpl.ob;
   inv_list.show_weight = TRUE;
   inv_list.weight_limit = 0;
@@ -1144,31 +1166,40 @@ void command_show (const char *params) {
 
 void update_list_labels (itemlist *l)
 {
-    char buf[MAX_BUF];
+    char weight[MAX_BUF];
+    char max_weight[MAX_BUF];
   
     /* draw title and put stuff in widgets */
-  
-    strcpy(buf, l->title);
-    gtk_label_set (GTK_LABEL(l->label), buf);
+
+    if ( strcmp( l->title, l->last_title ) ) {
+        strcpy(l->last_title, l->title);
+        strcpy(weight,l->title);
+        gtk_label_set (GTK_LABEL(l->label), weight);
+        gtk_widget_draw (l->label, NULL);
+    }
 
     if(l->env->weight < 0 || !l->show_weight) {
-	gtk_label_set (GTK_LABEL(l->weightlabel), " ");
-	gtk_label_set (GTK_LABEL(l->maxweightlabel), " ");
+	    strcpy(weight, " ");
+	    strcpy(max_weight, " ");
     }
     else if (!l->weight_limit) {
-	sprintf (buf, "%6.1f",l->env->weight);
-	gtk_label_set (GTK_LABEL(l->weightlabel), buf);
-	gtk_label_set (GTK_LABEL(l->maxweightlabel), " ");
+	    sprintf (weight, "%6.1f",l->env->weight);
+        strcpy (max_weight, " ");
     } else {
-	sprintf (buf, "%6.1f",l->env->weight);
-	gtk_label_set (GTK_LABEL(l->weightlabel), buf);
-	sprintf (buf, "/ %4d",l->weight_limit / 1000);
-	gtk_label_set (GTK_LABEL(l->maxweightlabel), buf);
+	    sprintf (weight, "%6.1f",l->env->weight);
+	    sprintf (max_weight, "/ %4d",l->weight_limit / 1000);
     }
 
-    gtk_widget_draw (l->label, NULL);
-    gtk_widget_draw (l->weightlabel, NULL);
-    gtk_widget_draw (l->maxweightlabel, NULL);
+    if ( strcmp( weight, l->last_weight ) ) {
+        strcpy(l->last_weight, weight);
+    	gtk_label_set (GTK_LABEL(l->weightlabel), weight);
+        gtk_widget_draw (l->weightlabel, NULL);
+    }
+    if ( strcmp( max_weight, l->last_maxweight ) ) {
+        strcpy(l->last_maxweight, max_weight);
+	    gtk_label_set (GTK_LABEL(l->maxweightlabel), max_weight);
+        gtk_widget_draw (l->maxweightlabel, NULL);
+    }
     
     l->env->inv_updated = FALSE;
 }
@@ -1178,7 +1209,7 @@ void update_list_labels (itemlist *l)
  *  
  *  (Maybe somewhat more often; look_list doesn't always have a weight shown, possibly.)
  */
-static void update_lists_labels ()
+static void update_lists_labels(void)
 {
   if (inv_list.env->inv_updated) {
     update_list_labels (&inv_list);
@@ -1202,23 +1233,6 @@ void set_weight_limit (uint32 wlim)
     update_list_labels(&inv_list);
 }
 
- 
-
-/* Called when CONFIG_COLORINV is changed. */
-
-static void set_colorinv_one(gpointer view_x, gpointer unused) {
-    inventory_viewer * view = (inventory_viewer *)view_x;
-
-    inventory_viewer_set_highlight(view, use_config[CONFIG_COLORINV]);
-}
-
-void inventory_update_colorinv(void) {
-    inventory_viewer_set_highlight(look_viewer, use_config[CONFIG_COLORINV]);
-            
-    g_list_foreach(highlit_inv_viewers, set_colorinv_one, NULL);
-}
-
-
 /* toggle weight */
 
 static void set_show_weight_inv_one(gpointer view_x, gpointer new_setting) {
@@ -1302,10 +1316,11 @@ void set_look_list_env(item * op) {
 }
 
 
-void open_container (item *op) 
-{
+void open_container (item *op) {
   set_look_list_env(op); 
   sprintf (look_list.title, "%s:", op->d_name);
+  gtk_widget_set_sensitive(closebutton, TRUE);
+
   update_list_labels (&look_list);
 }
 
@@ -1316,6 +1331,7 @@ void close_container(item *op)
 	client_send_apply (look_list.env->tag);
     set_look_list_env(cpl.below);
     strcpy (look_list.title, "You see:");
+    gtk_widget_set_sensitive(closebutton, FALSE);
     update_list_labels (&look_list);
   }
 }
@@ -1331,6 +1347,8 @@ static void close_container_callback(item *op)
     client_send_apply (look_list.env->tag);
     set_look_list_env(cpl.below);
     strcpy (look_list.title, "You see:");
+    gtk_widget_set_sensitive(closebutton, FALSE);
+
     update_list_labels (&look_list);
   }
 }
