@@ -1,5 +1,5 @@
-char *rcsid_gtk_png_c =
-    "$Id: png.c,v 1.11 2005/05/29 16:42:19 ryo_saeba Exp $";
+const char *rcsid_gtk_png_c =
+    "$Id: png.c,v 1.13 2006/02/23 20:18:28 akirschbaum Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -399,7 +399,7 @@ uint8 *rescale_rgba_data(uint8 *data, int *width, int *height, int scale)
 }
 
 
-guchar rgb[512*512*3];	/* Make this especially big to support larger images in the future */
+static guchar rgb[512*512*3];	/* Make this especially big to support larger images in the future */
 
 /* This takes data that has already been converted into RGBA format (via
  * png_to_data above perhaps) and creates a GdkPixmap and GdkBitmap out
@@ -456,7 +456,7 @@ int rgba_to_gdkpixmap(GdkWindow *window, uint8 *data,int width, int height,
 }
 
 
-int png_to_gdkpixmap(GdkWindow *window, uint8 *data, int len,
+static int png_to_gdkpixmap(GdkWindow *window, uint8 *data, int len,
 		   GdkPixmap **pix, GdkBitmap **mask, GdkColormap *colormap)
 {
     static uint8 *pixels=NULL;
