@@ -1,5 +1,5 @@
 char *rcsid_gtk_sound_c =
-    "$Id: sound.c,v 1.11 2005/02/14 05:42:01 mwedel Exp $";
+    "$Id: sound.c,v 1.13 2006/02/23 20:18:28 akirschbaum Exp $";
 
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -44,19 +44,19 @@ char *rcsid_gtk_sound_c =
 /* Got a pipe signal.  As of now, only thing we are piped to is the
  * sound client.
  */
-void signal_pipe(int i) {
+static void signal_pipe(int i) {
     /* do nothing, but perhaps do something more in the future */
 }
 
-FILE *sound_pipe=NULL;
-ChildProcess* sound_process;
+static FILE *sound_pipe = NULL;
+static ChildProcess *sound_process;
 /* init_sounds open the audio device, and reads any configuration files
  * that need to be.  It returns 0 on success.  On failure, the calling
  * function will likely disable sound support/requests from the server.
  */
 
 #ifndef WIN32
-int init_sounds()
+int init_sounds(void)
 {
     char sound_path[MAX_BUF];
 
