@@ -1,5 +1,5 @@
 char *rcsid_gtk_text_c =
-    "$Id: text.c,v 1.6 2005/05/29 15:34:45 tchize Exp $";
+    "$Id: text.c,v 1.11 2006/02/23 20:18:28 akirschbaum Exp $";
 /*
     Crossfire client, a client program for the crossfire program.
 
@@ -32,19 +32,20 @@ char *rcsid_gtk_text_c =
 #include "client.h"
 #include "gx11.h"
 #include "gtkproto.h"
-char* NO_TITLE="[no title]";
+static char NO_TITLE[] = "[no title]";
 
 #include "pixmaps/sign_flat.xpm"
 #include "pixmaps/sign_west.xpm"
 #include "pixmaps/sign_east.xpm"
+#include "pixmaps/close.xpm"
 GtkWidget* book_root = NULL;
 GtkWidget* book_notes = NULL;
 GdkBitmap* btnClose_bm = NULL;
 GdkPixmap* btnClose_pm = NULL;
 
 typedef struct picture_message_struct {
-    char* title;
-    char** xpm;
+    const char *title;
+    const char *const *xpm;
     int x;
     int y;
     int width;
@@ -55,36 +56,39 @@ typedef struct picture_message_struct {
     
 } picture_message;
 
-picture_message sign_message[] = {
+static picture_message sign_message[] = {
     {"sign",sign_flat_xpm,70,45,390,305,500,500,NULL},
     {"left sign",sign_west_xpm,95,85,615,190,750,400,NULL},
     {"right sign",sign_east_xpm,45,85,615,190,750,400,NULL},
     {"direction sign",sign_flat_xpm,70,45,390,305,500,500,NULL} };
-void prepare_book_window(){
-    if (!book_root){
-        #include "pixmaps/close.xpm"
+static void init_pictures(GtkWidget *refWindow) {
+    if (btnClose_pm==NULL)          
+        btnClose_pm = gdk_pixmap_create_from_xpm_d(refWindow->window,&btnClose_bm,
+                &gtk_widget_get_style(refWindow)->bg[GTK_STATE_NORMAL],
+                (gchar**)close_xpm);
+}
+static void prepare_book_window(void) {
+    if (!book_root){    
         book_root= gtk_window_new (GTK_WINDOW_TOPLEVEL);
         gtk_window_set_title(GTK_WINDOW(book_root),"books");
         book_notes = gtk_notebook_new();
         gtk_notebook_set_tab_pos(GTK_NOTEBOOK(book_notes),GTK_POS_LEFT);
         gtk_container_add(GTK_CONTAINER(book_root),book_notes);
         gtk_widget_show(GTK_WIDGET(book_notes));  
-        gtk_widget_show(GTK_WIDGET(book_root));              
-        btnClose_pm = gdk_pixmap_create_from_xpm_d(book_root->window,&btnClose_bm,
-                &gtk_widget_get_style(book_root)->bg[GTK_STATE_NORMAL],
-                (gchar**)close_xpm);
-            
-        gtk_signal_connect (GTK_OBJECT (book_root), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &book_root);      
+        gtk_widget_show(GTK_WIDGET(book_root));
+        init_pictures (book_root);
+        gtk_signal_connect (GTK_OBJECT (book_root), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &book_root);
         gtk_window_set_default_size(GTK_WINDOW(book_root),500,600);
         gtk_window_set_position(GTK_WINDOW(book_root),GTK_WIN_POS_CENTER);
     }
 }
-GtkWidget* create_text_picture_window(picture_message* layout, char* message){
-    GtkWidget *window, *content, *fixed, *scroll;    
+static GtkWidget *create_text_picture_window(picture_message *layout, char *message) {
+    GtkWidget *window, *content, *fixed, *scroll, *close;    
     window = gtk_window_new (GTK_WINDOW_DIALOG);
     gtk_window_set_title(GTK_WINDOW(window),layout->title);
     gtk_widget_set_app_paintable(window,TRUE);
     gtk_widget_realize(window);
+    init_pictures(window);
     if (layout->picture == NULL){
         layout->picture = gdk_pixmap_create_from_xpm_d(window->window,NULL,
                 &gtk_widget_get_style(window)->bg[GTK_STATE_NORMAL],
@@ -110,6 +114,9 @@ GtkWidget* create_text_picture_window(picture_message* layout, char* message){
     gtk_widget_show (scroll);
     gtk_widget_set_usize(scroll,layout->width,layout->height);
     gtk_fixed_put(GTK_FIXED(fixed),scroll,layout->x,layout->y);
+    close=gtk_button_new();    
+    gtk_widget_set_usize(close,0,0);
+    gtk_fixed_put(GTK_FIXED(fixed),close,0,0);
     gtk_widget_show(fixed);
     gtk_widget_show(content);
     
@@ -118,9 +125,14 @@ GtkWidget* create_text_picture_window(picture_message* layout, char* message){
             NULL,
             gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scroll))
         );
+	
+    gtk_signal_connect_object (GTK_OBJECT (close), "clicked",
+                   GTK_SIGNAL_FUNC(gtk_widget_destroy),
+                   GTK_OBJECT (window));    
+    gtk_widget_grab_focus (GTK_WIDGET(close));
     return window;
 }
-void show_media_message(const char* title, const char* message){
+static void show_media_message(const char *title, const char *message) {
     GtkWidget *window, *scroll, *content;
     window = gtk_window_new (GTK_WINDOW_DIALOG);
     gtk_window_set_title(GTK_WINDOW(window),message);
@@ -151,7 +163,7 @@ void show_media_message(const char* title, const char* message){
  * Parse message, extract multimedia information, and push
  * as appropriate in the GtkText
  */
-char *arcane_medium_fontname[]={
+static const char *const arcane_medium_fontname[] = {
     "-*-cuneifontlight-*-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-linotext-*-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-blackforest-*-r-*-*-*-120-*-*-*-*-iso8859-*",
@@ -160,7 +172,7 @@ char *arcane_medium_fontname[]={
     "-*-caligula-*-*-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *hand_medium_fontname[]={
+static const char *const hand_medium_fontname[] = {
     "-*-dobkinscript-*-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-coronetscript-*-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-muriel-*-r-*-*-*-120-*-*-*-*-iso8859-*",
@@ -169,61 +181,61 @@ char *hand_medium_fontname[]={
     "-*-rechtmanscript-*-r-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
 };
-char *strange_medium_fontname[]={
+static const char *const strange_medium_fontname[] = {
     "-*-annstone-*-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-shalomstick-*-r-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     
 };
-char *print_medium_fontname[]={
+static const char *const print_medium_fontname[] = {
     "-*-arial-medium-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-bookman-light-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-agate-normal-r-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *print_bold_fontname[]={
+static const char *const print_bold_fontname[] = {
     "-*-arial-bold-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-bookman-demi-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-agate-bold-r-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *print_italic_fontname[]={
+static const char *const print_italic_fontname[] = {
     "-*-arial-medium-i-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-bookman-light-i-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-agate-normal-i-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *print_italicbold_fontname[]={
+static const char *const print_italicbold_fontname[] = {
     "-*-arial-bold-i-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-bookman-demi-i-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-agate-bold-i-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *fixed_medium_fontname[]={
+static const char *const fixed_medium_fontname[] = {
     "-*-fixed-medium-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-courrier-medium-*-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-andale mono-medium-*-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *fixed_bold_fontname[]={
+static const char *const fixed_bold_fontname[] = {
     "-*-fixed-bold-r-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-courrier-bold-*-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-andale mono-medium-*-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *fixed_italic_fontname[]={
+static const char *const fixed_italic_fontname[] = {
     "-*-fixed-medium-o-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-courrier-medium-o-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-andale mono-medium-*-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-char *fixed_italicbold_fontname[]={
+static const char *const fixed_italicbold_fontname[] = {
     "-*-fixed-bold-i-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-courrier-bold-o-*-*-*-120-*-*-*-*-iso8859-*",
     "-*-andale mono-*-*-*-*-*-120-*-*-*-*-iso8859-*",
     NULL
     };
-GdkFont* load_a_font(char **font_list){
+static GdkFont *load_a_font(const char *const font_list[]) {
     GdkFont* result;
     int i;
     for (i=0; font_list[i]!=NULL;i++){
@@ -235,16 +247,16 @@ GdkFont* load_a_font(char **font_list){
     }
     return NULL;
 }
-GdkFont* style_fixed[4];
-GdkFont* style_arcane[4];
-GdkFont* style_hand[4];
-GdkFont* style_strange[4];
-GdkFont* style_print[4];
+static GdkFont *style_fixed[4];
+static GdkFont *style_arcane[4];
+static GdkFont *style_hand[4];
+static GdkFont *style_strange[4];
+static GdkFont *style_print[4];
 #define STYLE_BOLD 1
 #define STYLE_ITALIC 2
-int style_inited = 0;
+static int style_inited = 0;
 
-void init_fonts(){
+static void init_fonts(void) {
     if (!style_inited){
         style_fixed[0]=load_a_font(fixed_medium_fontname);
         style_fixed[1]=load_a_font(fixed_bold_fontname);
@@ -324,14 +336,14 @@ media_state write_media_with_state(GtkText* textarea, const char* message, media
     free(original);
     return current_state;
 }
-void add_book(char* title, char* message){
+static void add_book(char *title, char *message) {
     GtkWidget *content,*label,*hbox, *scroll, *panel, *close, *closepic;
     prepare_book_window();
     
     content = gtk_text_new(NULL,NULL);
     gtk_text_set_editable(GTK_TEXT(content),FALSE);
     gtk_text_set_word_wrap(GTK_TEXT(content),FALSE);
-    gtk_text_set_line_wrap(GTK_TEXT(content),FALSE);
+    gtk_text_set_line_wrap(GTK_TEXT(content),TRUE);
     write_media(GTK_TEXT(content),message);
     
     panel = gtk_vbox_new(FALSE,0);
@@ -348,9 +360,9 @@ void add_book(char* title, char* message){
     gtk_box_pack_start (GTK_BOX (panel), hbox, TRUE, TRUE, 0);
     
     label = gtk_label_new(title);
-//    tab = gtk_hbox_new();
-//    button_pic = gtk_pixmap_new(btnClose_pm, btnClose_bm);
-//    button = gtk_button_new();
+/*    tab = gtk_hbox_new();*/
+/*    button_pic = gtk_pixmap_new(btnClose_pm, btnClose_bm);*/
+/*    button = gtk_button_new();*/
     
     gtk_notebook_append_page(GTK_NOTEBOOK(book_notes),panel,label);
     gtk_widget_show(content);    
@@ -364,13 +376,19 @@ void add_book(char* title, char* message){
     
     gtk_notebook_set_page(GTK_NOTEBOOK(book_notes),gtk_notebook_page_num(GTK_NOTEBOOK(book_notes),panel));
     gdk_window_raise (book_root->window);
-    
+    gtk_widget_grab_focus (GTK_WIDGET(close));
     gtk_signal_connect_object (GTK_OBJECT (close), "clicked",
                    GTK_SIGNAL_FUNC(gtk_widget_destroy),
                    GTK_OBJECT (panel));    
        
 }
-void book_callback(int flag, int type, int subtype, char* message){
+/* we need access to those when a sign is auto applied. 
+ * We don't want to show player a new window while his character
+ * keeps running in background
+ */
+extern GtkWidget* gtkwin_info_text;
+extern GtkWidget* gtkwin_info_text2;
+static void book_callback(int flag, int type, int subtype, char *message) {
     LOG(LOG_DEBUG,"gtk::book_callback","got callback %d subtype %d\n",type,subtype);
     if (message!=NULL){
         char* title = message;
@@ -383,12 +401,19 @@ void book_callback(int flag, int type, int subtype, char* message){
         if (*message=='\0'){
             message=title;
             title=NO_TITLE;
-        }  
-        add_book(title,message);
+        }
+        if (!want_config[CONFIG_POPUPS]) /*autoapply*/{
+            if (use_config[CONFIG_SPLITINFO])
+                write_media(GTK_TEXT(gtkwin_info_text2),message);
+            else
+                write_media(GTK_TEXT(gtkwin_info_text),message);
+	} else
+            add_book(title,message);
+	
     }    
 }
-char* last_motd=NULL;
-void motd_callback(int flag, int type, int subtype, char* message){
+static char *last_motd = NULL;
+static void motd_callback(int flag, int type, int subtype, char *message) {
     
     if(last_motd)
         free(last_motd);
@@ -397,19 +422,17 @@ void motd_callback(int flag, int type, int subtype, char* message){
         LOG(LOG_ERROR,"gtk::motd_callback","Outa memory, no save of motd");
     else
         strcpy(last_motd,message);    
+
+    if (!want_config[CONFIG_POPUPS] || last_motd == NULL) {
+        write_media(GTK_TEXT(gtkwin_info_text), message);
+    }
 }
-void void_callback(int flag, int type, int subtype, char* message){
+static void void_callback(int flag, int type, int subtype, const char *message) {
     
     LOG(LOG_INFO,"gtk::void_callback","got message --\n%s\n",message);
     
 }
-/* we need access to those when a sign is auto applied. 
- * We don't want to show player a new window while his character
- * keeps running in background
- */
-extern GtkWidget* gtkwin_info_text;
-extern GtkWidget* gtkwin_info_text2;
-void sign_callback(int flag, int type, int subtype, char* message){
+static void sign_callback(int flag, int type, int subtype, char *message) {
     GtkWidget *window;
     int flags;
     if ( (subtype>4) || (subtype <1))    
@@ -421,7 +444,7 @@ void sign_callback(int flag, int type, int subtype, char* message){
         return;
     message++;
     
-    if (flags&0x01) /*autoapply*/{
+    if ((!want_config[CONFIG_POPUPS]) || (!want_config[CONFIG_SIGNPOPUP]) ||(flags&0x01)) /*autoapply*/{
         if (use_config[CONFIG_SPLITINFO])
             write_media(GTK_TEXT(gtkwin_info_text2),message);
         else
@@ -433,18 +456,18 @@ void sign_callback(int flag, int type, int subtype, char* message){
     }
     
 }
-char* getMOTD(){    
+const char *getMOTD(void) {
     return last_motd==NULL?"Please read motd written\nin [i]green[/i] inside main\nmessage window":last_motd;
 }
-char *rules = NULL;
+static char *rules = NULL;
 news_entry* first_news = NULL;
-char* get_rules(){
+const char *get_rules(void) {
     return rules;    
 }
-news_entry* get_news(){
+news_entry *get_news(void) {
     return first_news;    
 }
-void admin_callback(int flag, int type, int subtype, char* message){
+static void admin_callback(int flag, int type, int subtype, char *message) {
     char* str1;
     news_entry* new;
     switch (subtype){
@@ -489,14 +512,14 @@ void admin_callback(int flag, int type, int subtype, char* message){
     }
     show_media_message("Unknown admin message",message);
 }
-void init_text_callbacks(){
+void init_text_callbacks(void) {
     setTextManager(MSG_TYPE_BOOK,book_callback);
     setTextManager(MSG_TYPE_MOTD,motd_callback);
     setTextManager(MSG_TYPE_MONUMENT,void_callback);
     setTextManager(MSG_TYPE_SIGN,sign_callback);
     setTextManager(MSG_TYPE_ADMIN,admin_callback);
 }
-void cleanup_textmanagers(){
+void cleanup_textmanagers(void) {
     news_entry* last_entry;
     if (last_motd)
         free(last_motd);
